@@ -1,6 +1,5 @@
 #! /bin/sh
-# Please have a TMP or TMPDIR environment variable if you don't trust /tmp,
-# or don't run this as root.
+# Please have a TMP or TMPDIR environment variable if you don't trust /tmp.
 
 export PATH=/usr/local/bin:/usr/local/sbin:$PATH
 
@@ -64,34 +63,39 @@ $dialog \
 'without-iplogging' "Never log remote IP addresses (confidentiality)" off \
 'without-humor' "Disable humor (enabled by default)" off \
 'without-ascii' "Don't support 7-bits (ASCII) transfers" off \
-'without-banner' "Don't display the nice initial banner" off \
-'without-nonalnum' "Only allow minimal alpha-numeric characters" off \
 'without-globbing' "Don't include the globbing code" off \
+'without-nonalnum' "Only allow minimal alpha-numeric characters" off \
+'without-unicode' "Disable utf8 non-latin characters" off \
 'without-sendfile' "Don't use zero-copy optimizations" off \
+'without-privsep' "Disable privilege separation" off \
+'with-boring' "Display only boring messages" off \
 'with-brokenrealpath' "If your C library has a broken realpath()" off \
 'with-probe-random-dev' "To check for /dev/*random at run-time" off \
 'with-minimal' "Build only a minimal server for embedded systems" off \
 'with-paranoidmsg' "Use paranoid, but not admin-friendly messages" off \
 'with-sysquotas' "Use system (not virtual) quotas" off \
-'with-ldap' "Users database is an LDAP directory" off \
-'with-mysql' "Users database is a MySQL database" off \
 'with-altlog' "Support alternative log format (Apache-like)" on \
-'with-pam' 'Enable PAM authentication' off \
 'with-puredb' 'Support virtual (FTP-only) users' on \
 'with-extauth' 'Support external authentication modules' on \
+'with-ldap' "Users database is an LDAP directory" off \
+'with-mysql' "Users database is a MySQL database" off \
+'with-pgsql' "Users database is a PostgreSQL database" off \
+'with-pam' 'Enable PAM authentication' off \
 'with-cookie' "Support 'fortune' cookies" on \
 'with-throttling' "Support bandwidth throttling" on \
-'with-ftpwho' "Support the pure-ftpwho command" on \
 'with-ratios' "Support upload/download ratios" on \
 'with-quotas' "Support .ftpquota files" on \
+'with-ftpwho' "Support the pure-ftpwho command" on \
 'with-welcomemsg' "welcome.msg files backward compatibility" off \
-'with-largefile' "Support for files > 2 Gb (slower)" off \
-'with-uploadscript' "Allow running pure-uploadscript after upload" on \
+'with-uploadscript' "Allow running scripts after upload (experimental)" on \
 'with-virtualhosts' "Allow a distinct content for each IP address" on \
 'with-virtualchroot' "Follow symlinks outside a chroot jail" off \
 'with-diraliases' "Support directory aliases" on \
+'with-nonroot' "DO NOT USE unless you know what you are doing" off \
 'with-peruserlimits' "Support per-user concurrency limits" on \
-'with-privsep' "Enable privilege separation" off \
+'with-implicittls' "Implicit TLS (port 990) - DO NOT USE unless you know what you are doing" off \
+'with-tls' "Support SSL/TLS security layer" off \
+'with-bonjour' "Support Bonjour on MacOS X" off \
 2> $tmp
 
 cfgline='';
@@ -104,6 +108,7 @@ $dialog \
 --radiolist 'Choose a language for server messages' \
 20 78 10 \
 'english' "This is the default" on \
+'albanian' "Contributed by Moisi Xhaferaj" off \
 'german' "Contributed by Mathias Gumz" off \
 'romanian' "Contributed by Claudiu Costin" off \
 'french' "Contributed by Ping" off \
@@ -123,6 +128,8 @@ $dialog \
 'simplified-chinese' "Contributed by Hether Fygul" off \
 'czech' "Contributed by Martin Sarfy" off \
 'turkish' "Contributed by Mehmet Cokcevik" off \
+'hungarian' "Contributed by Banhalmi Csaba" off \
+'catalan' "Contributed by Oriol Magrano" off \
 2> $tmp
 
 z=$(cat $tmp)
