@@ -2,14 +2,6 @@
 
 #ifndef NO_STANDALONE
 
-/* Yes, I know, I could have used multi-hashed linked lists, 
- * binary trees and other funny stuff. It'd have been better, cleaner,
- * faster. Yeah, it'd have added many lines of complicated source code
- * to win 1/100000 sec on system with more than 500 sim users.
- * Great win, but sorry, I won't do that just to please theorical
- * algorithms wizards. -Jedi.
- */
-
 # include "ftpd.h"
 # include "dynamic.h"
 # include "ftpwho-update.h"
@@ -24,8 +16,8 @@ static IPTrack *iptrack_list;
 void iptrack_delete_pid(const pid_t pid)
 {
     unsigned int c = 0U;
-    
-    if (iptrack_list == NULL) { 
+
+    if (iptrack_list == NULL) {
         return;
     }
     do {
@@ -34,7 +26,7 @@ void iptrack_delete_pid(const pid_t pid)
             return;
         }
         c++;
-    } while (c < maxusers);    
+    } while (c < maxusers);
 }
 
 void iptrack_free(void)
@@ -47,17 +39,17 @@ static unsigned int iptrack_find_ip_or_shift
     (const struct sockaddr_storage * const ip)
 {
     unsigned int c = 0U;
-    
+
     do {
-        if (iptrack_list[c].pid != (pid_t) 0 && 
+        if (iptrack_list[c].pid != (pid_t) 0 &&
             STORAGE_FAMILY(iptrack_list[c].ip) != STORAGE_FAMILY(*ip)) {
             if (STORAGE_FAMILY(iptrack_list[c].ip) == AF_INET &&
-                STORAGE_SIN_ADDR(iptrack_list[c].ip) == STORAGE_SIN_ADDR(*ip)) {
+                STORAGE_SIN_ADDR_CONST(iptrack_list[c].ip) == STORAGE_SIN_ADDR_CONST(*ip)) {
                 return c;
             } else if (STORAGE_FAMILY(iptrack_list[c].ip) == AF_INET6 &&
                        IN6_ARE_ADDR_EQUAL
-                       (&STORAGE_SIN_ADDR6_NF(iptrack_list[c].ip),
-                        &STORAGE_SIN_ADDR6_NF(*ip))) {
+                       (&STORAGE_SIN_ADDR6_NF_CONST(iptrack_list[c].ip),
+                        &STORAGE_SIN_ADDR6_NF_CONST(*ip))) {
                 return c;
             }
         }
@@ -75,20 +67,20 @@ unsigned int iptrack_get(const struct sockaddr_storage * const ip)
 {
     unsigned int c = 0U;
     unsigned int nb = 0U;
-    
-    if (iptrack_list == NULL) { 
+
+    if (iptrack_list == NULL) {
         return 0U;
     }
     do {
-        if (iptrack_list[c].pid != (pid_t) 0 && 
+        if (iptrack_list[c].pid != (pid_t) 0 &&
             STORAGE_FAMILY(iptrack_list[c].ip) == STORAGE_FAMILY(*ip)) {
             if (STORAGE_FAMILY(iptrack_list[c].ip) == AF_INET &&
-                STORAGE_SIN_ADDR(iptrack_list[c].ip) == STORAGE_SIN_ADDR(*ip)) {
+                STORAGE_SIN_ADDR_CONST(iptrack_list[c].ip) == STORAGE_SIN_ADDR_CONST(*ip)) {
                 nb++;
             } else if (STORAGE_FAMILY(iptrack_list[c].ip) == AF_INET6 &&
                        IN6_ARE_ADDR_EQUAL
-                       (&STORAGE_SIN_ADDR6_NF(iptrack_list[c].ip),
-                        &STORAGE_SIN_ADDR6_NF(*ip))) {
+                       (&STORAGE_SIN_ADDR6_NF_CONST(iptrack_list[c].ip),
+                        &STORAGE_SIN_ADDR6_NF_CONST(*ip))) {
                 nb++;
             }
         }
@@ -102,10 +94,10 @@ void iptrack_add(const struct sockaddr_storage * const ip,
                  const pid_t pid)
 {
     unsigned int c = 0U;
-    
+
     if (iptrack_list == NULL) {
         unsigned int ci = 0U;
-        
+
         if ((iptrack_list = malloc(maxusers * sizeof *iptrack_list)) == NULL) {
             return;
         }
@@ -113,7 +105,7 @@ void iptrack_add(const struct sockaddr_storage * const ip,
             iptrack_list[ci].pid = (pid_t) 0;
             ci++;
         } while (ci < maxusers);
-    }     
+    }
     do {
         if (iptrack_list[c].pid == (pid_t) 0U) {
             force:
@@ -122,7 +114,7 @@ void iptrack_add(const struct sockaddr_storage * const ip,
             return;
         }
         c++;
-    } while (c < maxusers);    
+    } while (c < maxusers);
     c = iptrack_find_ip_or_shift(ip);
     goto force;
 }
