@@ -24,21 +24,21 @@ static void antiidle(void)
         if ((time(NULL) - noopidle) > (time_t) idletime_noop) {
             die(421, LOG_INFO, MSG_TIMEOUT_NOOP, (unsigned long) idletime_noop);
         }
-    }    
+    }
 }
 
-/* 
- * Introduce a random delay, to avoid guessing existing user names by
+/*
+ * Introduce a random delay, to mitigate guessing existing user names by
  * mesuring delay. It's especially true when LDAP is used.
  * No need to call usleep2() because we are root at this point.
  */
 
 static void randomdelay(void)
 {
-    usleep(rand() % 15000UL);          /* dummy... no need for arc4 */
+    usleep(rand() % 15000UL);
 }
 
-/* 
+/*
  * Simple but fast command-line reader. We break the FTP protocol here,
  * because we deny access to files with strange characters in their name.
  * Now, I seriously doubt that clients should be allowed to upload files
@@ -48,22 +48,24 @@ static void randomdelay(void)
  * results on some filesystems, etc. So control chars are replaced by "_".
  * Better be safe than 100% RFC crap compliant but unsafe. If you really want
  * RFC compliance, define RFC_CONFORMANT_PARSER. But I will hate you.
- * 
+ *
  * RFC_CONFORMANT_LINES is another thing that clients should implement
  * properly (and it's trivial to do) : lines must be ended with \r\n .
- * Guess what ? 
+ * Guess what ?
  * Some broken clients are just sending \n ... Grrrrrrrrrrrr !!!!!!!!!!!!!!!
- * 
+ *
  * -Frank.
  */
 
 static size_t scanned;
 static size_t readnbd;
 
+#ifdef WITH_TLS
 static void flush_cmd(void)
 {
     scanned = readnbd = (size_t) 0U;
 }
+#endif
 
 int sfgets(void)
 {
@@ -71,9 +73,9 @@ int sfgets(void)
     int pollret;
     ssize_t readnb;
     signed char seen_r = 0;
-    
+
     if (scanned > (size_t) 0U) {       /* support pipelining */
-        readnbd -= scanned;        
+        readnbd -= scanned;
         memmove(cmd, cmd + scanned, readnbd);   /* safe */
         scanned = (size_t) 0U;
     }
@@ -153,13 +155,13 @@ int sfgets(void)
             }
 #else
             /* replace control chars with _ */
-            cmd[scanned] = '_';                
+            cmd[scanned] = '_';
 #endif
         }
         scanned++;
     }
     die(421, LOG_WARNING, MSG_LINE_TOO_LONG);   /* don't remove this */
-    
+
     return 0;                         /* to please GCC */
 }
 
@@ -172,7 +174,7 @@ static char *revealextraspc(char * const s_)
 {
     unsigned char *s = (unsigned char *) s_;
     unsigned char *sn;
-    
+
     if (s == NULL) {
         return s_;
     }
@@ -188,45 +190,34 @@ static char *revealextraspc(char * const s_)
         sn++;
     } while (*sn != 0U);
     do {
-        sn--;        
+        sn--;
         if (!isspace(*sn)) {
             break;
         }
         *sn = '_';
     } while (sn != s);
-    
+
     return s_;
 }
 #endif
 
-#ifdef WITH_RFC2640
-char *charset_client2fs(const char * const string)
+#ifndef MINIMAL
+static void parse_file_time_change(char *arg)
 {
-    char *output = NULL, *output_;
-    size_t inlen, outlen, outlen_;
-    
-    inlen = strlen(string);
-    outlen_ = outlen = inlen * (size_t) 4U + (size_t) 1U;
-    if (outlen <= inlen ||
-        (output_ = output = calloc(outlen, (size_t) 1U)) == NULL) {
-        die_mem();
+    char *name;
+
+    if ((name = strchr(arg, ' ')) == NULL) {
+        addreply_noformat(501, MSG_MISSING_ARG);
+        return;
     }
-    if (utf8 > 0 && strcasecmp(charset_fs, "utf-8") != 0) {
-        if (iconv(iconv_fd_utf82fs, (char **) &string,
-                  &inlen, &output_, &outlen_) == (size_t) -1) {
-            strncpy(output, string, outlen);
-        }
-    } else if (utf8 <= 0 && strcasecmp(charset_fs, charset_client) != 0) {
-        if (iconv(iconv_fd_client2fs, (char **) &string,
-                  &inlen, &output_, &outlen_) == (size_t) -1) {
-            strncpy(output, string, outlen);
-        }
-    } else {
-        strncpy(output, string, outlen);
+    while (*name == ' ') {
+        name++;
     }
-    output[outlen - 1] = 0;    
-    
-    return output;
+    if (*name == 0) {
+        addreply_noformat(501, MSG_MISSING_ARG);
+        return;
+    }
+    doutime(name, arg);
 }
 #endif
 
@@ -235,9 +226,6 @@ void parser(void)
     char *arg;
 #ifndef MINIMAL
     char *sitearg;
-#endif
-#ifdef WITH_RFC2640
-    char *narg = NULL;
 #endif
     size_t n;
 
@@ -277,7 +265,7 @@ void parser(void)
         }
 #endif
         n = (size_t) 0U;
-        while ((isalpha((unsigned char) cmd[n]) || cmd[n] == '@') &&
+        while ((isalnum((unsigned char) cmd[n]) || cmd[n] == '@') &&
                n < cmdsize) {
             cmd[n] = (char) tolower((unsigned char) cmd[n]);
             n++;
@@ -287,14 +275,13 @@ void parser(void)
         }
         if (n == (size_t) 0U) {
             nop:
-            addreply_noformat(500, "?");
             continue;
         }
-#ifdef SKIP_COMMAND_TRAILING_SPACES        
+#ifdef SKIP_COMMAND_TRAILING_SPACES
         while (isspace((unsigned char) cmd[n]) && n < cmdsize) {
             cmd[n++] = 0;
         }
-        arg = cmd + n;        
+        arg = cmd + n;
         while (cmd[n] != 0 && n < cmdsize) {
             n++;
         }
@@ -321,17 +308,14 @@ void parser(void)
                    cmd, strcmp(cmd, "pass") ? arg : "<*>");
 #endif
         }
-#ifdef WITH_RFC2640
-        narg = charset_client2fs(arg);
-	arg = narg;
-#endif
+
         /*
          * antiidle() is called with dummy commands, usually used by clients
          * who are wanting extra idle time. We give them some, but not too much.
          * When we jump to wayout, the idle timer is not zeroed. It means that
          * we didn't issue an 'active' command like RETR.
          */
-        
+
 #ifndef MINIMAL
         if (!strcmp(cmd, "noop")) {
             antiidle();
@@ -342,7 +326,8 @@ void parser(void)
         if (!strcmp(cmd, "user")) {
 #ifdef WITH_TLS
             if (enforce_tls_auth > 1 && tls_cnx == NULL) {
-                die(421, LOG_WARNING, MSG_TLS_NEEDED);
+                addreply(421, MSG_TLS_NEEDED);
+                goto wayout;
             }
 #endif
             douser(arg);
@@ -363,7 +348,7 @@ void parser(void)
             addreply_noformat(215, "UNIX Type: L8");
             goto wayout;
 #ifdef WITH_TLS
-        } else if (enforce_tls_auth > 0 &&
+        } else if (enforce_tls_auth > 0 && loggedin == 0 &&
                    !strcmp(cmd, "auth") && !strcasecmp(arg, "tls")) {
             addreply_noformat(234, "AUTH TLS OK.");
             doreply();
@@ -374,6 +359,17 @@ void parser(void)
             goto wayout;
         } else if (!strcmp(cmd, "pbsz")) {
             addreply_noformat(tls_cnx == NULL ? 503 : 200, "PBSZ=0");
+        } else if (!strcmp(cmd, "ccc")) {
+            if (loggedin == 0 || tls_cnx == NULL) {
+                addreply_noformat(534, "CCC not allowed at this point");
+                goto wayout;
+            }
+            addreply_noformat(200, "Control connection unencrypted");
+            doreply();
+            tls_close_session(&tls_cnx);
+            tls_cnx = NULL;
+            flush_cmd();
+            goto wayout;
         } else if (!strcmp(cmd, "prot")) {
             if (tls_cnx == NULL) {
                 addreply_noformat(503, MSG_PROT_BEFORE_PBSZ);
@@ -416,21 +412,29 @@ void parser(void)
             dotype(arg);
             goto wayout;
         } else if (!strcmp(cmd, "mode")) {
-            antiidle();                
+            antiidle();
             domode(arg);
             goto wayout;
 #ifndef MINIMAL
         } else if (!strcmp(cmd, "feat")) {
             dofeat();
             goto wayout;
-	} else if (!strcmp(cmd, "opts")) {
-	    doopts(arg);
-	    goto wayout;
+        } else if (!strcmp(cmd, "opts")) {
+            doopts(arg);
+            goto wayout;
 #endif
         } else if (!strcmp(cmd, "stru")) {
             dostru(arg);
             goto wayout;
 #ifndef MINIMAL
+        } else if (!strcmp(cmd, "connect") ||
+                   !strcmp(cmd, "delete") ||
+                   !strcmp(cmd, "get") ||
+                   !strcmp(cmd, "head") ||
+                   !strcmp(cmd, "options") ||
+                   !strcmp(cmd, "post") ||
+                   !strcmp(cmd, "put")) {
+            die(500, LOG_INFO, "HTTP command: [%s]", cmd);
         } else if (!strcmp(cmd, "help")) {
             goto help_site;
 #endif
@@ -439,8 +443,8 @@ void parser(void)
             debug++;
             addreply(200, MSG_XDBG_OK, debug);
             goto wayout;
-#endif            
-        } else if (loggedin == 0) {            
+#endif
+        } else if (loggedin == 0) {
             /* from this point, all commands need authentication */
             addreply_noformat(530, MSG_NOT_LOGGED_IN);
             goto wayout;
@@ -461,11 +465,11 @@ void parser(void)
             } else if (!strcmp(cmd, "estp")) {
                 doestp();
 #endif
-            } else if (disallow_passive == 0 && 
+            } else if (disallow_passive == 0 &&
                        (!strcmp(cmd, "pasv") || !strcmp(cmd, "p@sw"))) {
                 dopasv(0);
-            } else if (disallow_passive == 0 && 
-                       (!strcmp(cmd, "epsv") && 
+            } else if (disallow_passive == 0 &&
+                       (!strcmp(cmd, "epsv") &&
                        (broken_client_compat == 0 ||
                         STORAGE_FAMILY(ctrlconn) == AF_INET6))) {
                 if (!strcasecmp(arg, "all")) {
@@ -476,35 +480,36 @@ void parser(void)
                 } else {
                     dopasv(1);
                 }
-#ifndef MINIMAL            
-            } else if (disallow_passive == 0 && !strcmp(cmd, "spsv")) {
-                dopasv(2);
+#ifndef MINIMAL
+            } else if (!strcmp(cmd, "pret")) {
+                struct stat st;
+
+                if (strncasecmp("RETR ", arg, sizeof "RETR " - 1U) == 0) {
+                    arg += sizeof "RETR " - 1U;
+                    if (*arg == 0 || access(arg, R_OK) != 0 ||
+                        stat(arg, &st) != 0 || S_ISREG(st.st_mode) == 0) {
+                        addreply_noformat(550, MSG_FILE_DOESNT_EXIST);
+                        goto wayout;
+                    }
+                }
+                addreply_noformat(200, MSG_READY_TO_PROCEED);
             } else if (!strcmp(cmd, "allo")) {
                 if (*arg == 0) {
                     addreply_noformat(501, MSG_STAT_FAILURE);
                 } else {
                     const off_t size = (off_t) strtoull(arg, NULL, 10);
-                    
+
                     if (size < (off_t) 0) {
-                        addreply_noformat(501, MSG_STAT_FAILURE);                        
+                        addreply_noformat(501, MSG_STAT_FAILURE);
                     } else {
                         doallo(size);
                     }
                 }
 #endif
             } else if (!strcmp(cmd, "pwd") || !strcmp(cmd, "xpwd")) {
-#ifdef WITH_RFC2640
-		char *nwd;
-#endif
                 antiidle();
-#ifdef WITH_RFC2640
-		nwd = charset_fs2client(wd);
-		addreply(257, "\"%s\" " MSG_IS_YOUR_CURRENT_LOCATION, nwd);
-		free(nwd);
-#else
                 addreply(257, "\"%s\" " MSG_IS_YOUR_CURRENT_LOCATION, wd);
-#endif
-                goto wayout;                
+                goto wayout;
             } else if (!strcmp(cmd, "cdup") || !strcmp(cmd, "xcup")) {
                 docwd("..");
             } else if (!strcmp(cmd, "retr")) {
@@ -544,7 +549,7 @@ void parser(void)
                     if (enforce_tls_auth == 3 &&
                         data_protection_level != CPL_PRIVATE) {
                         addreply_noformat(521, MSG_PROT_PRIVATE_NEEDED);
-                    } else 
+                    } else
 #endif
                     {
                         dostor(arg, 0, autorename);
@@ -559,7 +564,7 @@ void parser(void)
                     if (enforce_tls_auth == 3 &&
                         data_protection_level != CPL_PRIVATE) {
                         addreply_noformat(521, MSG_PROT_PRIVATE_NEEDED);
-                    } else 
+                    } else
 #endif
                     {
                         dostor(arg, 1, 0);
@@ -573,10 +578,10 @@ void parser(void)
                 if (enforce_tls_auth == 3 &&
                     data_protection_level != CPL_PRIVATE) {
                     addreply_noformat(521, MSG_PROT_PRIVATE_NEEDED);
-                } else 
+                } else
 #endif
                 {
-            	     dostou();
+                     dostou();
                 }
 #endif
 #ifndef DISABLE_MKD_RMD
@@ -597,16 +602,12 @@ void parser(void)
 #ifndef MINIMAL
             } else if (!strcmp(cmd, "stat")) {
                 if (*arg != 0) {
-                    modern_listings = 0;
-                    donlist(arg, 1, 1, 1, 1);
+                    dolist(arg, 1);
                 } else {
-                    addreply_noformat(211, "http://www.pureftpd.org/");
+                    addreply_noformat(211, "https://www.pureftpd.org/");
                 }
 #endif
             } else if (!strcmp(cmd, "list")) {
-#ifndef MINIMAL
-                modern_listings = 0;
-#endif
 #ifdef WITH_TLS
                 if (enforce_tls_auth == 3 &&
                     data_protection_level != CPL_PRIVATE) {
@@ -614,12 +615,9 @@ void parser(void)
                 } else
 #endif
                 {
-                    donlist(arg, 0, 1, 0, 1);
+                    dolist(arg, 0);
                 }
             } else if (!strcmp(cmd, "nlst")) {
-#ifndef MINIMAL                
-                modern_listings = 0;
-#endif
 #ifdef WITH_TLS
                 if (enforce_tls_auth == 3 &&
                     data_protection_level != CPL_PRIVATE) {
@@ -627,29 +625,30 @@ void parser(void)
                 } else
 #endif
                 {
-                    donlist(arg, 0, 0, 0, broken_client_compat);
+                    donlst(arg);
                 }
 #ifndef MINIMAL
+            } else if (!strcmp(cmd, "mfmt")) {
+                parse_file_time_change(arg);
             } else if (!strcmp(cmd, "mlst")) {
-#ifdef WITH_TLS
+# ifdef WITH_TLS
                 if (enforce_tls_auth == 3 &&
                     data_protection_level != CPL_PRIVATE) {
                     addreply_noformat(521, MSG_PROT_PRIVATE_NEEDED);
                 } else
-#endif
+# endif
                 {
                     domlst(*arg != 0 ? arg : ".");
                 }
             } else if (!strcmp(cmd, "mlsd")) {
-                modern_listings = 1;
-#ifdef WITH_TLS
+# ifdef WITH_TLS
                 if (enforce_tls_auth == 3 &&
                     data_protection_level != CPL_PRIVATE) {
                     addreply_noformat(521, MSG_PROT_PRIVATE_NEEDED);
                 } else
-#endif
+# endif
                 {
-                    donlist(arg, 0, 1, 1, 0);
+                    domlsd(*arg != 0 ? arg : ".");
                 }
 #endif
             } else if (!strcmp(cmd, "abor")) {
@@ -685,7 +684,7 @@ void parser(void)
                     dositetime();
                 } else if (!strcasecmp(arg, "help")) {
                     help_site:
-                    
+
                     addreply_noformat(214, MSG_SITE_HELP CRLF
 # ifdef WITH_DIRALIASES
                                       " ALIAS" CRLF
@@ -695,7 +694,7 @@ void parser(void)
                 } else if (!strcasecmp(arg, "chmod")) {
                     char *sitearg2;
                     mode_t mode;
-                    
+
                     parsechmod:
                     if (sitearg == NULL || *sitearg == 0) {
                         addreply_noformat(501, MSG_MISSING_ARG);
@@ -704,10 +703,10 @@ void parser(void)
                     sitearg2 = sitearg;
                     while (*sitearg2 != 0 && !isspace((unsigned char) *sitearg2)) {
                         sitearg2++;
-                    }                    
+                    }
                     while (*sitearg2 != 0 && isspace((unsigned char) *sitearg2)) {
                         sitearg2++;
-                    }                    
+                    }
                     if (*sitearg2 == 0) {
                         addreply_noformat(550, MSG_NO_FILE_NAME);
                         goto chmod_wayout;
@@ -722,19 +721,19 @@ void parser(void)
                     (void) 0;
                 } else if (!strcasecmp(arg, "utime")) {
                     char *sitearg2;
-                    
+
                     if (sitearg == NULL || *sitearg == 0) {
                         addreply_noformat(501, MSG_NO_FILE_NAME);
                         goto utime_wayout;
-                    }		    
+                    }
                     if ((sitearg2 = strrchr(sitearg, ' ')) == NULL ||
                         sitearg2 == sitearg) {
                         addreply_noformat(501, MSG_MISSING_ARG);
                         goto utime_wayout;
                     }
                     if (strcasecmp(sitearg2, " UTC") != 0) {
-                        addreply_noformat(500, "UTC Only");
-                        goto utime_wayout;			
+                        parse_file_time_change(sitearg);
+                        goto utime_wayout;
                     }
                     *sitearg2-- = 0;
                     if ((sitearg2 = strrchr(sitearg, ' ')) == NULL ||
@@ -755,18 +754,18 @@ void parser(void)
                     }
                     *sitearg2++ = 0;
                     if (*sitearg2 == 0) {
-                        goto utime_no_arg;			
+                        goto utime_no_arg;
                     }
                     doutime(sitearg, sitearg2);
                     utime_wayout:
                     (void) 0;
-# ifdef WITH_DIRALIASES		    
+# ifdef WITH_DIRALIASES
                 } else if (!strcasecmp(arg, "alias")) {
                     if (sitearg == NULL || *sitearg == 0) {
                         print_aliases();
                     } else {
                         const char *alias;
-                        
+
                         if ((alias = lookup_alias(sitearg)) != NULL) {
                             addreply(214, MSG_ALIASES_ALIAS, sitearg, alias);
                         } else {
@@ -808,10 +807,6 @@ void parser(void)
         }
         noopidle = (time_t) -1;
         wayout:
-#ifdef WITH_RFC2640
-        free(narg);
-        narg = NULL;
-#endif
 #ifdef THROTTLING
         if (throttling_delay != 0UL) {
             usleep2(throttling_delay);
