@@ -9,15 +9,9 @@
 #endif
 
 #include <stdio.h>
-#ifdef STDC_HEADERS
-# include <stdlib.h>
-# include <stddef.h>
-# include <stdarg.h>
-#else
-# if HAVE_STDLIB_H
-#  include <stdlib.h>
-# endif
-#endif
+#include <stdlib.h>
+#include <stddef.h>
+#include <stdarg.h>
 #include <sys/types.h>
 #ifdef HAVE_STDINT_H
 # include <stdint.h>
@@ -42,16 +36,10 @@
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
-#ifdef TIME_WITH_SYS_TIME
+#if HAVE_SYS_TIME_H
 # include <sys/time.h>
-# include <time.h>
-#else
-# if HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
 #endif
+#include <time.h>
 #include <sys/stat.h>
 #ifdef HAVE_FCNTL_H
 # include <fcntl.h>
@@ -105,9 +93,6 @@
 #ifdef HAVE_UTIME_H
 # include <utime.h>
 #endif
-#ifdef WITH_RFC2640
-# include <iconv.h>
-#endif
 
 #include "mysnprintf.h"
 
@@ -277,7 +262,7 @@ typedef struct AuthResult_ {
     unsigned long throttling_bandwidth_ul;
     unsigned long throttling_bandwidth_dl;
     int throttling_ul_changed;
-    int throttling_dl_changed;    
+    int throttling_dl_changed;
 #endif
 #ifdef QUOTAS
     unsigned long long user_quota_size;
@@ -335,7 +320,6 @@ void docwd(const char *dir);
 void doretr(char *name);
 void dorest(const char *name);
 void dodele(char *name);
-void doallo(off_t size);
 void dostor(char *name, const int append, const int autorename);
 void domkd(char *name);
 void dormd(char *name);
@@ -363,18 +347,18 @@ void dornto(char *name);
 void dostou(void);
 void dofeat(void);
 void domlst(const char * const file);
+void domlsd(const char * const base);
 void dositetime(void);
 int ul_check_free_space(const char *name, const double min_space);
-void mappedtov4(struct sockaddr_storage *ss);
 void disablesignals(void);
 void getnames(void);
-void donlist(char *arg, const int on_ctrlconn, const int opt_l_,
-             const int opt_a_, const int split_args);
+void dolist(char *arg, const int on_ctrlconn);
+void donlst(const char *base);
 void opendata(void);
 void closedata(void);
 void client_fflush(void);
 void client_printf(const char * const format, ...)
-    __attribute__ ((format(printf, 1, 2)));    
+    __attribute__ ((format(printf, 1, 2)));
 void addreply(const int code, const char * const line, ...)
     __attribute__ ((format(printf, 2, 3)));
 void addreply_noformat(const int code, const char * const line);
@@ -382,7 +366,7 @@ void doreply(void);
 void sighandler(int sig);
 void prevent(char *arg);
 unsigned int daemons(in_port_t server_port);
-void logfile(const int facility, const char *format, ...) 
+void logfile(const int facility, const char *format, ...)
     __attribute__ ((format(printf, 2, 3)));
 char *skip_telnet_controls(const char *str);
 void die(const int err, const int priority, const char * const format, ...)
@@ -399,9 +383,6 @@ unsigned int zrand(void);
 void simplify(char *subdir);
 int checkprintable(register const char *s);
 void delete_atomic_file(void);
-#ifdef WITH_RFC2640
-char *charset_fs2client(const char * const string);
-#endif
 void usleep2(const unsigned long microsec);
 int mysnprintf(char *str, size_t size, const char *format, ...);
 
@@ -416,9 +397,24 @@ extern int opt_a, opt_C, opt_d, opt_F, opt_l, opt_R;
 #endif
 
 #ifdef WITH_TLS
-# ifndef TLS_CERTIFICATE_FILE
-#  define TLS_CERTIFICATE_FILE "/etc/ssl/private/pure-ftpd.pem"
+# ifndef TLS_CONFDIR
+#  define TLS_CONFDIR "/etc/ssl/private"
 # endif
+# ifndef TLS_CERTIFICATE_FILE
+#  define TLS_CERTIFICATE_FILE TLS_CONFDIR "/pure-ftpd.pem"
+# endif
+# ifndef TLS_KEY_FILE
+#  define TLS_KEY_FILE TLS_CERTIFICATE_FILE
+# endif
+#endif
+
+#ifndef TLS_DEFAULT_CIPHER_SUITE
+# define TLS_DEFAULT_CIPHER_SUITE \
+    "ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:DHE-RSA-AES128-GCM-SHA256:DHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256:ECDHE-ECDSA-AES128-SHA:ECDHE-RSA-AES256-SHA384:ECDHE-RSA-AES128-SHA:ECDHE-ECDSA-AES256-SHA384:ECDHE-ECDSA-AES256-SHA:ECDHE-RSA-AES256-SHA:DHE-RSA-AES128-SHA256:DHE-RSA-AES128-SHA:DHE-RSA-AES256-SHA256:DHE-RSA-AES256-SHA:AES128-GCM-SHA256:AES256-GCM-SHA384:AES128-SHA256:AES256-SHA256:AES128-SHA:AES256-SHA:!DSS"
+#endif
+
+#ifndef TLS_DEFAULT_ECDH_CURVE
+# define TLS_DEFAULT_ECDH_CURVE "prime256v1"
 #endif
 
 #define FAKE_SHELL "ftp"
@@ -447,6 +443,14 @@ extern int opt_a, opt_C, opt_d, opt_F, opt_l, opt_R;
 # endif
 #endif
 
+#ifndef CERTD_PID_FILE
+# ifdef NON_ROOT_FTP
+#  define CERTD_PID_FILE CONFDIR "/pure-certd.pid"
+# else
+#  define CERTD_PID_FILE STATEDIR "/run/pure-certd.pid"
+# endif
+#endif
+
 #ifndef NON_ROOT_FTP
 # ifdef IMPLICIT_TLS
 #  define DEFAULT_FTP_PORT_S "990"
@@ -469,7 +473,7 @@ extern int opt_a, opt_C, opt_d, opt_F, opt_l, opt_R;
 
 /*
  * Some operating systems (at least Solaris > 2.7 and FreeBSD) have strange
- * troubles with reusing TCP ports, even when SO_REUSEADDR is enabled. 
+ * troubles with reusing TCP ports, even when SO_REUSEADDR is enabled.
  * As a workaround, we try several unassigned privileged ports.
  * The last way is to let the OS assign a port.
  * For egress filtering, you can accept connections from ports <= 20
@@ -480,21 +484,24 @@ extern int opt_a, opt_C, opt_d, opt_F, opt_l, opt_R;
     DEFAULT_FTP_DATA_PORT, 2U, 3U, 4U, 5U, 6U, 10U, 14U, 16U, 0U \
 }
 
-#ifndef MAXPATHLEN
-# ifdef PATH_MAX
-#  define MAXPATHLEN PATH_MAX
+#ifndef PATH_MAX
+# ifdef MAXPATHLEN
+#  define PATH_MAX MAXPATHLEN
 # else
-#  define MAXPATHLEN 65536U
-Warning: neither PATH_MAX nor MAXPATHLEN were found.
-Remove these lines if you really want to compile the server, but
-the server may be insecure if a wrong value is set here.    
+#  define PATH_MAX 65536U
+#  error Warning: neither PATH_MAX nor MAXPATHLEN were found.
+#  error Remove these lines if you really want to compile the server, but
+#  error the server may be insecure if a wrong value is set here.
 # endif
 #endif
-#if (MAXPATHLEN) >= (INT_MAX)
-Your platform has a very large MAXPATHLEN, we should not trust it.
+#if (PATH_MAX) >= (INT_MAX)
+Your platform has a very large PATH_MAX, we should not trust it.
 #endif
 
-#ifndef DEFAULT_MAX_USERS    
+#ifndef MAX_PASSWORD_LEN
+# define MAX_PASSWORD_LEN 512
+#endif
+#ifndef DEFAULT_MAX_USERS
 # define DEFAULT_MAX_USERS 50
 #endif
 #ifndef DEFAULT_FTP_DATA_PORT
@@ -505,7 +512,7 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 # endif
 #endif
 #ifndef MAX_SYSLOG_LINE
-# define MAX_SYSLOG_LINE (MAXPATHLEN + 512U)
+# define MAX_SYSLOG_LINE (PATH_MAX + 512U)
 #endif
 #ifndef DEFAULT_IDLE
 # define DEFAULT_IDLE (15UL * 60UL)
@@ -513,7 +520,7 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 #ifndef MAX_SITE_IDLE
 # define MAX_SITE_IDLE (42UL * 60UL)
 #endif
-#ifndef DEFAULT_MAX_LS_FILES    
+#ifndef DEFAULT_MAX_LS_FILES
 # define DEFAULT_MAX_LS_FILES 10000U
 #endif
 #ifndef DEFAULT_MAX_LS_DEPTH
@@ -525,7 +532,7 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 #ifndef MAX_CPU_TIME
 # define MAX_CPU_TIME (4 * 60 * 60)           /* Max allowed CPU time per session */
 #endif
-#ifndef MAX_SESSION_XFER_IDLE    
+#ifndef MAX_SESSION_XFER_IDLE
 # define MAX_SESSION_XFER_IDLE (24 * 60 * 60)   /* Max duration of a transfer */
 #endif
 #ifndef MAX_USER_LENGTH
@@ -538,10 +545,14 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 # define DEFAULT_FACILITY LOG_LOCAL2
 #endif
 
-#if defined(WITH_LDAP) || defined(WITH_MYSQL) || defined(WITH_PGSQL)
-# define MAX_DATA_SIZE (16 * 1024 * 1024)       /* Max memory usage - SQL/LDAP need more */
-#else
-# define MAX_DATA_SIZE (8 * 1024 * 1024)       /* Max memory usage */
+#ifndef MAX_DATA_SIZE
+# ifdef HAVE_LIBSODIUM
+#  define MAX_DATA_SIZE (70 * 1024 * 1024)
+# elif defined(WITH_LDAP) || defined(WITH_MYSQL) || defined(WITH_PGSQL)
+#  define MAX_DATA_SIZE (16 * 1024 * 1024)       /* Max memory usage - SQL/LDAP need more */
+# else
+#  define MAX_DATA_SIZE (8 * 1024 * 1024)       /* Max memory usage */
+# endif
 #endif
 
 #if CONF_TCP_SO_RCVBUF < 65536
@@ -552,7 +563,7 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 # undef CONF_TCP_SO_SNDBUF
 # define CONF_TCP_SO_SNDBUF 65536
 #endif
-    
+
 #ifndef DL_MIN_CHUNK_SIZE
 # define DL_MIN_CHUNK_SIZE (8 * 1024)
 #endif
@@ -590,18 +601,24 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 #endif
 #ifndef UL_MAX_CHUNK_SIZE
 # define UL_MAX_CHUNK_SIZE (512 * 1024UL)
-#endif    
+#endif
 
-#define VHOST_PREFIX_MAX_LEN 64    
+#define VHOST_PREFIX_MAX_LEN 64
 
-#define PUREFTPD_TMPFILE_PREFIX ".pureftpd-"    
+#define PUREFTPD_TMPFILE_PREFIX ".pureftpd-"
 #define ATOMIC_PREFIX_PREFIX PUREFTPD_TMPFILE_PREFIX "upload."
+#define WIN32_ANON_DIR "/ftp"
 
 #define STORAGE_PORT(X)  (*storage_port(&(X)))
+#define STORAGE_PORT_CONST(X)  (*storage_port_const(&(X)))
 #define STORAGE_PORT6(X) (*storage_port6(&(X)))
+#define STORAGE_PORT6_CONST(X) (*storage_port6_const(&(X)))
 #define STORAGE_SIN_ADDR(X) (storage_sin_addr(&(X))->s_addr)
+#define STORAGE_SIN_ADDR_CONST(X) (storage_sin_addr_const(&(X))->s_addr)
 #define STORAGE_SIN_ADDR6(X) (storage_sin_addr6(&(X))->s6_addr)
+#define STORAGE_SIN_ADDR6_CONST(X) (storage_sin_addr6_const(&(X))->s6_addr)
 #define STORAGE_SIN_ADDR6_NF(X) (*(storage_sin_addr6(&(X))))
+#define STORAGE_SIN_ADDR6_NF_CONST(X) (*(storage_sin_addr6_const(&(X))))
 
 #ifdef HAVE_SS_LEN
 # define STORAGE_LEN(X) ((X).ss_len)
@@ -664,7 +681,7 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 #ifdef HAVE_NDIR_H
 # include <ndir.h>
 #endif
-       
+
 #ifdef STAT_MACROS_BROKEN
 # undef S_ISBLK
 # undef S_ISCHR
@@ -677,7 +694,7 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 # undef S_ISREG
 # undef S_ISSOCK
 #endif                            /* STAT_MACROS_BROKEN.  */
-    
+
 #ifndef S_IFMT
 # define S_IFMT 0170000
 #endif
@@ -709,11 +726,11 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 #if !defined(S_ISNWK) && defined(S_IFNWK)    /* HP/UX */
 # define S_ISNWK(m) (((m) & S_IFMT) == S_IFNWK)
 #endif
-    
+
 #ifndef S_IEXEC
 # define S_IEXEC S_IXUSR
 #endif
-    
+
 #ifndef S_IXUSR
 # define S_IXUSR S_IEXEC
 #endif
@@ -732,7 +749,7 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 #else
 # define ISCTRLCODE(X) ((X) == 0x7f || ((unsigned char) (X)) < 32U)
 #endif
-    
+
 #ifndef HAVE_MUNMAP
 # define munmap(A, B) (0)
 #endif
@@ -760,7 +777,7 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 # endif
 #endif
 
-#if defined(HAVE_DEV_ARANDOM) || defined(HAVE_DEV_URANDOM) || defined(HAVE_DEV_RANDOM)
+#if defined(HAVE_DEV_URANDOM) || defined(HAVE_DEV_RANDOM)
 # define HAVE_RANDOM_DEV 1
 #endif
 
@@ -772,9 +789,9 @@ Your platform has a very large MAXPATHLEN, we should not trust it.
 
 #ifndef offsetof
 # define offsetof(type, member) ((size_t) &((type*) NULL)->member)
-#endif    
-    
-#include "bsd-realpath.h"    
+#endif
+
+#include "bsd-realpath.h"
 #include "fakechroot.h"
-        
+
 #endif
