@@ -43,7 +43,7 @@
 #define DIRSCAN_FAILURE_DELAY (100000UL)  /* Delay after each chdir failure */
 #define ASCII_CHUNKSIZE 65536U
 #define BANNER_MAXLINES 100
-#define MAX_SERVER_REPLY_LEN (MAXPATHLEN + (size_t) 50U)
+#define MAX_SERVER_REPLY_LEN (PATH_MAX + (size_t) 50U)
 
 #ifndef NO_STANDALONE
 static volatile sig_atomic_t stop_server;
@@ -51,6 +51,7 @@ static const char *standalone_port = DEFAULT_FTP_PORT_S;
 static const char *standalone_ip;
 static volatile unsigned int nb_children;
 static volatile int listenfd = -1;
+static volatile int listenfd6 = -1;
 #endif
 
 struct reply {
@@ -59,7 +60,8 @@ struct reply {
 };
 
 static const char *GETOPT_OPTIONS =
-    "014Aa:bc:"
+    "0146"
+    "Aa:bc:"
 #ifndef NO_STANDALONE
     "BC:"
 #endif
@@ -74,7 +76,7 @@ static const char *GETOPT_OPTIONS =
 #ifdef QUOTAS
     "n:"
 #endif
-#if defined(WITH_UPLOAD_SCRIPT)
+#ifdef WITH_UPLOAD_SCRIPT
     "o"
 #endif
 #ifdef WITH_ALTLOG
@@ -91,28 +93,40 @@ static const char *GETOPT_OPTIONS =
 #ifdef THROTTLING
     "t:T:"
 #endif
-    "u:U:V:wWxX"
+    "u:U:"
+#ifdef WITH_VIRTUAL_HOSTS
+    "V:"
+#endif
+    "wWxX"
+#ifdef WITH_OSX_BONJOUR
+    "v:"
+#endif
 #ifdef PER_USER_LIMITS
     "y:"
 #endif
 #ifdef WITH_TLS
-    "Y:"
-#endif    
+    "2:3:Y:J:"
+#endif
     "zZ";
 
-#ifndef NO_GETOPT_LONG    
+#ifndef NO_GETOPT_LONG
 static struct option long_options[] = {
-    { "notruncate", 0, NULL, '0' },    
+    { "notruncate", 0, NULL, '0' },
     { "logpid", 0, NULL, '1' },
+# ifdef WITH_TLS
+    { "certfile", 1, NULL, '2' },
+    { "extcert", 1, NULL, '3' },
+# endif
     { "ipv4only", 0, NULL, '4' },
+    { "ipv6only", 0, NULL, '6' },
     { "chrooteveryone", 0, NULL, 'A' },
     { "trustedgid", 1, NULL, 'a' },
     { "brokenclientscompatibility", 0, NULL, 'b' },
 # ifndef NO_STANDALONE
-    { "daemonize", 0, NULL, 'B' },                
+    { "daemonize", 0, NULL, 'B' },
     { "maxclientsperip", 1, NULL, 'C' },
 # endif
-    { "maxclientsnumber", 1, NULL, 'c' },    
+    { "maxclientsnumber", 1, NULL, 'c' },
     { "verboselog", 0, NULL, 'd' },
     { "displaydotfiles", 0, NULL, 'D' },
     { "anonymousonly", 0, NULL, 'e' },
@@ -124,13 +138,13 @@ static struct option long_options[] = {
 # ifndef NO_STANDALONE
     { "pidfile", 1, NULL, 'g' },
 # endif
-    { "norename", 0, NULL, 'G' },    
+    { "norename", 0, NULL, 'G' },
     { "help", 0, NULL, 'h' },
     { "dontresolve", 0, NULL, 'H' },
     { "maxidletime", 1, NULL, 'I' },
     { "anonymouscantupload", 0, NULL, 'i' },
     { "createhomedir", 0, NULL, 'j' },
-    { "keepallfiles", 0, NULL, 'K' },    
+    { "keepallfiles", 0, NULL, 'K' },
     { "maxdiskusagepct", 1, NULL, 'k' },
     { "login", 1, NULL, 'l' },
     { "limitrecursion", 1, NULL, 'L' },
@@ -140,7 +154,7 @@ static struct option long_options[] = {
 # ifdef QUOTAS
     { "quota", 1, NULL, 'n' },
 # endif
-# if defined(WITH_UPLOAD_SCRIPT)
+# ifdef WITH_UPLOAD_SCRIPT
     { "uploadscript", 0, NULL, 'o' },
 # endif
 # ifdef WITH_ALTLOG
@@ -153,16 +167,23 @@ static struct option long_options[] = {
     { "userratio", 1, NULL, 'Q' },
 # endif
     { "autorename", 0, NULL, 'r' },
-    { "nochmod", 0, NULL, 'R' },    
+    { "nochmod", 0, NULL, 'R' },
     { "antiwarez", 0, NULL, 's' },
 # ifndef NO_STANDALONE
     { "bind", 1, NULL, 'S' },
 # endif
+# ifdef THROTTLING
     { "anonymousbandwidth", 1, NULL, 't' },
     { "userbandwidth", 1, NULL, 'T' },
+# endif
     { "umask", 1, NULL, 'U' },
     { "minuid", 1, NULL, 'u' },
+# ifdef WITH_VIRTUAL_HOSTS
     { "trustedip", 1, NULL, 'V' },
+# endif
+# ifdef WITH_OSX_BONJOUR
+    { "bonjour", 1, NULL, 'v' },
+# endif
     { "allowuserfxp", 0, NULL, 'w' },
     { "allowanonymousfxp", 0, NULL, 'W' },
     { "prohibitdotfileswrite", 0, NULL, 'x' },
@@ -172,6 +193,7 @@ static struct option long_options[] = {
 # endif
 # ifdef WITH_TLS
     { "tls", 1, NULL, 'Y' },
+    { "tlsciphersuite", 1, NULL, 'J' },
 # endif
     { "allowdotfiles", 0, NULL, 'z' },
     { "customerproof", 0, NULL, 'Z' },
@@ -185,6 +207,7 @@ static const AltLogPrefixes altlogprefixes[] = {
     { "clf", ALTLOG_CLF },
     { "stats", ALTLOG_STATS },
     { "w3c", ALTLOG_W3C },
+    { "xferlog", ALTLOG_XFERLOG },
     { NULL, ALTLOG_NONE }
 };
 
@@ -192,7 +215,7 @@ static const AltLogPrefixes altlogprefixes[] = {
 # define ALTLOG_DEFAULT ALTLOG_CLF
 #endif
 
-#ifdef WITH_PRIVSEP
+#ifndef WITHOUT_PRIVSEP
 # define VERSION_PRIVSEP " [privsep]"
 #else
 # define VERSION_PRIVSEP ""
@@ -204,21 +227,19 @@ static const AltLogPrefixes altlogprefixes[] = {
 # define VERSION_TLS ""
 #endif
 
-#ifndef HAVE_SYS_FSUID_H
 static sigset_t old_sigmask;
-#endif
 
 #ifndef NO_PROCNAME_CHANGE
 # if defined(__linux__) && !defined(HAVE_SETPROCTITLE)
 static char **argv0;
-static int argv_lth;
+static size_t argv_lth;
 # endif
 #endif
 
 /*
  * An authentication handler has three functions:
  * - One - parse() - is called with an optional file name, that contains a
- *   configuration file, or whatever is passed in the -l command-line switch 
+ *   configuration file, or whatever is passed in the -l command-line switch
  *   for this authentication.
  * - Another one, check() is called when the user has entered his password.
  *   It should fill an AuthResult structure.
@@ -273,12 +294,65 @@ typedef struct Authentications_ {
 
 static Authentications *first_authentications, *last_authentications;
 
+typedef struct DLHandler_ {
+    int clientfd;
+    void *tls_clientfd;
+    int xferfd;
+    int f;
+    void *tls_fd;
+    off_t file_size;
+    size_t dlmap_size;
+    off_t cur_pos;
+    off_t chunk_size;
+    off_t min_chunk_size;
+    off_t default_chunk_size;
+    off_t max_chunk_size;
+    off_t dlmap_pos;
+    off_t dlmap_fdpos;
+    off_t total_downloaded;
+    size_t sizeof_map;
+    unsigned char *map;
+    unsigned char *map_data;
+    int ascii_mode;
+    double min_sleep;
+    double max_sleep;
+    unsigned long bandwidth;
+    struct pollfd pfds_f_in;
+} DLHandler;
+
+typedef struct ULHandler_ {
+    unsigned char *buf;
+    size_t sizeof_buf;
+    int clientfd;
+    void *tls_clientfd;
+    int xferfd;
+    void *tls_fd;
+    int f;
+    off_t cur_pos;
+    off_t chunk_size;
+    off_t min_chunk_size;
+    off_t default_chunk_size;
+    off_t max_chunk_size;
+    off_t total_uploaded;
+    int ascii_mode;
+    double min_sleep;
+    double max_sleep;
+    unsigned long bandwidth;
+    off_t max_filesize;
+    unsigned long idletime;
+    struct pollfd pfds[2];
+    struct pollfd pfds_command;
+} ULHandler;
+
+#define PFD_DATA 0
+#define PFD_COMMANDS 1
+
 #if defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined (__CYGWIN__)
 static struct passwd *fakegetpwnam(const char * const name);
 # define getpwnam(A) fakegetpwnam(A)
 # define getpwuid(A) fakegetpwnam(NULL)
-# define WIN32_ANON_DIR "/ftp"
 #endif
+#define NON_ROOT_ANON_DIR "/ftp"
 
 #ifdef PROBE_RANDOM_AT_RUNTIME
 static const char *random_device;
