@@ -269,6 +269,7 @@ static int pw_pgsql_connect(PGconn ** const id_sql_server)
     char *conninfo = NULL;
     size_t sizeof_conninfo;
     char *escaped_server = NULL;
+    char *escaped_port = NULL;
     char *escaped_db = NULL;
     char *escaped_user = NULL;
     char *escaped_pw = NULL;
@@ -277,6 +278,7 @@ static int pw_pgsql_connect(PGconn ** const id_sql_server)
     *id_sql_server = NULL;
 
     if ((escaped_server = pw_pgsql_escape_conninfo(server)) == NULL ||
+        (escaped_port = pw_pgsql_escape_conninfo(port)) == NULL ||
         (escaped_db = pw_pgsql_escape_conninfo(db)) == NULL ||
         (escaped_user = pw_pgsql_escape_conninfo(user)) == NULL ||
         (escaped_pw = pw_pgsql_escape_conninfo(pw)) == NULL) {
@@ -294,7 +296,8 @@ static int pw_pgsql_connect(PGconn ** const id_sql_server)
     }
     if (SNCHECK(snprintf(conninfo, sizeof_conninfo,
                          PGSQL_CONNECT_FMTSTRING,
-                         server, port, db, user, pw), sizeof_conninfo)) {
+                         escaped_server, escaped_port, escaped_db,
+                         escaped_user, escaped_pw), sizeof_conninfo)) {
         goto bye;
     }
     if ((*id_sql_server = PQconnectdb(conninfo)) == NULL ||
@@ -311,6 +314,7 @@ static int pw_pgsql_connect(PGconn ** const id_sql_server)
     bye:
     free(conninfo);
     free(escaped_server);
+    free(escaped_port);
     free(escaped_db);
     free(escaped_user);
     free(escaped_pw);
@@ -409,8 +413,7 @@ void pw_pgsql_check(AuthResult * const result,
     char *escaped_decimal_ip = NULL;
     char *scrambled_password = NULL;
     int committed = 1;
-    int crypto_scrypt = 0, crypto_crypt = 0, crypto_md5 = 0, crypto_sha1 = 0,
-        crypto_plain = 0;
+    int crypto_argon2 = 0, crypto_scrypt = 0, crypto_crypt = 0, crypto_plain = 0;
     unsigned long decimal_ip_num = 0UL;
     char decimal_ip[42];
     char hbuf[NI_MAXHOST];
@@ -499,22 +502,26 @@ void pw_pgsql_check(AuthResult * const result,
     }
     result->auth_ok--;                  /* -1 */
     if (strcasecmp(crypto, PASSWD_SQL_ANY) == 0) {
+        crypto_argon2++;
         crypto_scrypt++;
         crypto_crypt++;
-        crypto_md5++;
-        crypto_sha1++;
+    } else if (strcasecmp(crypto, PASSWD_SQL_ARGON2) == 0) {
+        crypto_argon2++;
     } else if (strcasecmp(crypto, PASSWD_SQL_SCRYPT) == 0) {
         crypto_scrypt++;
     } else if (strcasecmp(crypto, PASSWD_SQL_CRYPT) == 0) {
         crypto_crypt++;
-    } else if (strcasecmp(crypto, PASSWD_SQL_MD5) == 0) {
-        crypto_md5++;
-    } else if (strcasecmp(crypto, PASSWD_SQL_SHA1) == 0) {
-        crypto_sha1++;
     } else {                           /* default to plaintext */
         crypto_plain++;
     }
-#ifdef HAVE_LIBSODIUM
+#ifdef crypto_pwhash_STRPREFIX
+    if (crypto_argon2 != 0) {
+        if (crypto_pwhash_str_verify(spwd, password, strlen(password)) == 0) {
+            goto auth_ok;
+        }
+    }
+#endif
+#ifdef crypto_pwhash_scryptsalsa208sha256_STRPREFIX
     if (crypto_scrypt != 0) {
         if (crypto_pwhash_scryptsalsa208sha256_str_verify
             (spwd, password, strlen(password)) == 0) {
@@ -530,22 +537,6 @@ void pw_pgsql_check(AuthResult * const result,
             goto auth_ok;
         }
     }
-    if (crypto_md5 != 0) {
-        const char *crypted;
-
-        if ((crypted = (const char *) crypto_hash_md5(password, 1)) != NULL &&
-            pure_strcmp(crypted, spwd) == 0) {
-            goto auth_ok;
-        }
-    }
-    if (crypto_sha1 != 0) {
-        const char *crypted;
-
-        if ((crypted = (const char *) crypto_hash_sha1(password, 1)) != NULL &&
-            pure_strcmp(crypted, spwd) == 0) {
-            goto auth_ok;
-        }
-    }
     if (crypto_plain != 0) {
         if (*password != 0 &&    /* refuse null cleartext passwords */
             pure_strcmp(password, spwd) == 0) {
