@@ -1,12 +1,11 @@
 #ifndef __LOG_PGSQL_H__
 #define __LOG_PGSQL_H__ 1
 
+#define PASSWD_SQL_ARGON2 "argon2"
 #define PASSWD_SQL_SCRYPT "scrypt"
 #define PASSWD_SQL_CRYPT "crypt"
 #define PASSWD_SQL_CLEARTEXT "cleartext"
 #define PASSWD_SQL_PGSQL "password"
-#define PASSWD_SQL_MD5 "md5"
-#define PASSWD_SQL_SHA1 "sha1"
 #define PASSWD_SQL_ANY "any"
 #define PGSQL_DEFAULT_SERVER "localhost"
 #define PGSQL_DEFAULT_PORT 5432
