@@ -7,6 +7,7 @@
 #include "dynamic.h"
 #include "ftpwho-update.h"
 #include "globals.h"
+#include "safe_rw.h"
 #ifdef WITH_TLS
 # include "tls.h"
 #endif
@@ -20,19 +21,20 @@ static void wrstr(const int f, void * const tls_fd, const char *s)
     static char outbuf[CONF_TCP_SO_SNDBUF];
     static size_t outcnt;
     size_t l;
-    
+
     if (s == NULL) {
         if (outcnt > (size_t) 0U) {
 #ifdef WITH_TLS
             if (tls_fd != NULL) {
-                if (secure_safe_write(tls_fd, outbuf, outcnt) < 0) {
+                if (secure_safe_write(tls_fd, outbuf, outcnt) !=
+                    (ssize_t) outcnt) {
                     return;
-                } 
+                }
             } else
-#endif      
+#endif
             {
                 (void) tls_fd;
-                if (safe_write(f, outbuf, outcnt) != 0) {
+                if (safe_write(f, outbuf, outcnt, -1) != (ssize_t) outcnt) {
                     return;
                 }
             }
@@ -50,37 +52,41 @@ static void wrstr(const int f, void * const tls_fd, const char *s)
     }
     if (outcnt < sizeof outbuf) {
         const size_t rest = sizeof outbuf - outcnt;
-        
+
         memcpy(outbuf + outcnt, s, rest);   /* secure, see above */
         s += rest;
         l -= rest;
     }
 #ifdef WITH_TLS
-    if (data_protection_level == CPL_PRIVATE) {
-        if (secure_safe_write(tls_fd, outbuf, sizeof outbuf) < 0) {
+    if (tls_fd != NULL) {
+        if (secure_safe_write(tls_fd, outbuf, sizeof outbuf) !=
+            (ssize_t) sizeof outbuf) {
             return;
-        } 
+        }
     } else
 #endif
-    {       
-        if (safe_write(f, outbuf, sizeof outbuf) != 0) {
+    {
+        if (safe_write(f, outbuf, sizeof outbuf, -1) !=
+            (ssize_t) sizeof outbuf) {
             return;
         }
     }
 #ifdef WITH_TLS
-    if (data_protection_level == CPL_PRIVATE) {    
+    if (tls_fd != NULL) {
         while (l > sizeof outbuf) {
-            if (secure_safe_write(tls_fd, s, sizeof outbuf) < 0) {
+            if (secure_safe_write(tls_fd, s, sizeof outbuf) !=
+                (ssize_t) sizeof outbuf) {
                 return;
             }
             s += sizeof outbuf;
             l -= sizeof outbuf;
-        } 
+        }
     } else
 #endif
     {
         while (l > sizeof outbuf) {
-            if (safe_write(f, s, sizeof outbuf) != 0) {
+            if (safe_write(f, s, sizeof outbuf, -1) !=
+                (ssize_t) sizeof outbuf) {
                 return;
             }
             s += sizeof outbuf;
@@ -123,7 +129,7 @@ const char *getname(const uid_t uid)
     }
     if (
 # ifndef ALWAYS_RESOLVE_IDS
-        chrooted == 0 && 
+        chrooted == 0 &&
 # endif
         authresult.slow_tilde_expansion == 0) {
         pwd = getpwuid(uid);
@@ -136,19 +142,19 @@ const char *getname(const uid_t uid)
         die_mem();
     }
     if (pwd != NULL) {
-        if (SNCHECK(snprintf(p->name, (size_t) 11U, 
+        if (SNCHECK(snprintf(p->name, (size_t) 11U,
                              "%-10.10s", pwd->pw_name), (size_t) 11U)) {
             _EXIT(EXIT_FAILURE);
         }
     } else {
-        if (SNCHECK(snprintf(p->name, (size_t) 11U, "%-10d", uid), 
+        if (SNCHECK(snprintf(p->name, (size_t) 11U, "%-10d", uid),
                     (size_t) 11U)) {
             _EXIT(EXIT_FAILURE);
         }
     }
     p->next = user_head;
     user_head = p;
-    
+
     return p->name;
 }
 
@@ -162,9 +168,9 @@ const char *getgroup(const gid_t gid)
         if (p->gid == gid) {
             return p->name;
         }
-    } 
-# ifndef ALWAYS_RESOLVE_IDS   
-    if (chrooted == 0) 
+    }
+# ifndef ALWAYS_RESOLVE_IDS
+    if (chrooted == 0)
 # endif
     {
         pwd = getgrgid(gid);
@@ -182,14 +188,14 @@ const char *getgroup(const gid_t gid)
             _EXIT(EXIT_FAILURE);
         }
     } else {
-        if (SNCHECK(snprintf(p->name, (size_t) 11U, "%-10d", gid), 
+        if (SNCHECK(snprintf(p->name, (size_t) 11U, "%-10d", gid),
                     (size_t) 11U)) {
             _EXIT(EXIT_FAILURE);
         }
     }
     p->next = group_head;
     group_head = p;
-    
+
     return p->name;
 }
 #endif
@@ -226,48 +232,24 @@ static void addfile(const char *name, const char *suffix)
 }
 
 /* listfile returns non-zero if the file is a directory */
-static int listfile(const PureFileInfo * const fi,  const char *name)
+static int listfile(const PureFileInfo * const fi, const char *name)
 {
     int rval = 0;
     struct stat st;
     struct tm *t;
     char suffix[2] = { 0, 0 };
-    char m[MAXPATHLEN + 1U];    
+    char m[PATH_MAX + 1U];
+    const char *format;
 
-#ifndef MINIMAL
-    if (modern_listings != 0) {
-        const char *n;
-        char *alloca_nameline;
-        const size_t sizeof_nameline = MAXPATHLEN + 256U;
-
-        if (fi == NULL) {
-            n = name;
-        } else {
-            n = FI_NAME(fi);
-        }
-        if ((alloca_nameline = ALLOCA(sizeof_nameline)) == NULL) {
-            return 0;
-        }
-        if ((rval = modernformat(n, alloca_nameline,
-                                 sizeof_nameline, "")) < 0) {
-            ALLOCA_FREE(alloca_nameline);            
-            return 0;
-        }
-        addfile(alloca_nameline, suffix);
-        ALLOCA_FREE(alloca_nameline);
-        
-        return rval;
-    }
-#endif
     if (fi == NULL) {
         if (lstat(name, &st) < 0) {
             return 0;
-        }    
+        }
     } else {
         st.st_size = fi->size;
-        st.st_mtime = fi->mtime;        
+        st.st_mtime = fi->mtime;
         st.st_mode = fi->mode;
-        st.st_nlink = fi->nlink;        
+        st.st_nlink = fi->nlink;
         st.st_uid = fi->uid;
         st.st_gid = fi->gid;
         name = FI_NAME(fi);
@@ -286,7 +268,7 @@ static int listfile(const PureFileInfo * const fi,  const char *name)
 # endif
         S_ISLNK(st.st_mode)) {
         struct stat sts;
-        
+
         if (stat(name, &sts) == 0 && !S_ISLNK(sts.st_mode)) {
             st = sts;
         }
@@ -328,9 +310,9 @@ static int listfile(const PureFileInfo * const fi,  const char *name)
         }
         if (m[0] != ' ') {
             char *alloca_nameline;
-            const size_t sizeof_nameline = MAXPATHLEN + MAXPATHLEN + 128U;
+            const size_t sizeof_nameline = PATH_MAX + PATH_MAX + 128U;
             char timeline[6U];
-            
+
             if (st.st_mode & 0400) {
                 m[1] = 'r';
             }
@@ -363,7 +345,7 @@ static int listfile(const PureFileInfo * const fi,  const char *name)
                 m[9] = (char) (st.st_mode & 01000 ? 't' : 'x');
             } else if (st.st_mode & 01000) {
                 m[9] = 'T';
-            }            
+            }
             if (time(NULL) - st.st_mtime > 180 * 24 * 60 * 60) {
                 if (SNCHECK(snprintf(timeline, sizeof timeline, "%5d",
                                      t->tm_year + 1900), sizeof timeline)) {
@@ -378,11 +360,16 @@ static int listfile(const PureFileInfo * const fi,  const char *name)
             if ((alloca_nameline = ALLOCA(sizeof_nameline)) == NULL) {
                 return 0;
             }
+            if ((unsigned long long) st.st_size < 10000000000ULL) {
+                format = "%s %4u %s %s %10llu %s %2d %s %s";
+            } else {
+                format = "%s %4u %s %s %18llu %s %2d %s %s";
+            }
             if (SNCHECK(snprintf(alloca_nameline, sizeof_nameline,
-                                 "%s %4u %s %s %10llu %s %2d %s %s", 
+                                 format,
                                  m, (unsigned int) st.st_nlink,
                                  getname(st.st_uid),
-                                 getgroup(st.st_gid), 
+                                 getgroup(st.st_gid),
                                  (unsigned long long) st.st_size,
                                  months[t->tm_mon],
                                  t->tm_mday, timeline, name),
@@ -391,10 +378,10 @@ static int listfile(const PureFileInfo * const fi,  const char *name)
                 _EXIT(EXIT_FAILURE);
             }
             if (S_ISLNK(st.st_mode)) {
-                char *p = alloca_nameline + strlen(alloca_nameline);                
+                char *p = alloca_nameline + strlen(alloca_nameline);
                 {
                     ssize_t sx;
-                    
+
                     if ((sx = readlink(name, m, sizeof m - 1U)) > 0) {
                         m[sx] = 0;
                     } else {
@@ -412,19 +399,19 @@ static int listfile(const PureFileInfo * const fi,  const char *name)
                         suffix[0] = '*';
                     }
                 }
-                /* 2 * MAXPATHLEN + gap should be enough, but be paranoid... */
+                /* 2 * PATH_MAX + gap should be enough, but be paranoid... */
                 if (SNCHECK
                     (snprintf(p, (sizeof_nameline) - strlen(alloca_nameline),
-                              " -> %s", m), 
+                              " -> %s", m),
                      (sizeof_nameline) - strlen(alloca_nameline))) {
-                    ALLOCA_FREE(alloca_nameline);                    
+                    ALLOCA_FREE(alloca_nameline);
                     _EXIT(EXIT_FAILURE);
                 }
             }
             addfile(alloca_nameline, suffix);
-            ALLOCA_FREE(alloca_nameline);            
+            ALLOCA_FREE(alloca_nameline);
         }                    /* hide non-downloadable files */
-    } else {        
+    } else {
         if (S_ISREG(st.st_mode) ||
             S_ISDIR(st.st_mode) || S_ISLNK(st.st_mode)) {
             addfile(name, suffix);
@@ -438,9 +425,7 @@ static void outputfiles(int f, void * const tls_fd)
     unsigned int n;
     struct filename *p;
     struct filename *q;
-    char *c_buf; /* buffer with charset of client */
 
-    (void) c_buf;
     if (!head) {
         return;
     }
@@ -508,16 +493,11 @@ static void outputfiles(int f, void * const tls_fd)
                 pad[1] = '\n';
                 pad[2] = 0;
             }
-#ifdef WITH_RFC2640
-            c_buf = charset_fs2client(q->line);
-            wrstr(f, tls_fd, c_buf);
-            free(c_buf);
-#else
             wrstr(f, tls_fd, q->line);
-#endif
             wrstr(f, tls_fd, pad);
             q = q->right;
             free(tmp);
+            tmp = NULL;
         }
     }
 
@@ -529,19 +509,19 @@ static void outputfiles(int f, void * const tls_fd)
 
 /* functions to to sort for qsort() */
 static int cmp(const void * const a, const void * const b)
-{    
+{
     return strcmp(FI_NAME((const PureFileInfo *) a),
                   FI_NAME((const PureFileInfo *) b));
 }
 
 static int cmp_r(const void * const a, const void * const b)
-{    
+{
     return strcmp(FI_NAME((const PureFileInfo *) b),
                   FI_NAME((const PureFileInfo *) a));
 }
 
 static int cmp_t(const void * const a, const void * const b)
-{    
+{
     if (((const PureFileInfo *) a)->mtime < ((const PureFileInfo *) b)->mtime) {
         return 1;
     }
@@ -552,12 +532,12 @@ static int cmp_t(const void * const a, const void * const b)
 }
 
 static int cmp_rt(const void * const a, const void * const b)
-{    
+{
     return cmp_t(b, a);
 }
 
 static int cmp_S(const void * const a, const void * const b)
-{    
+{
     if (((const PureFileInfo *) a)->size < ((const PureFileInfo *) b)->size) {
         return 1;
     }
@@ -568,7 +548,7 @@ static int cmp_S(const void * const a, const void * const b)
 }
 
 static int cmp_rS(const void * const a, const void * const b)
-{    
+{
     return cmp_S(b, a);
 }
 
@@ -576,7 +556,7 @@ static PureFileInfo *sreaddir(char **names_pnt)
 {
     struct stat st;
     DIR *d;
-    struct dirent *de;    
+    struct dirent *de;
     PureFileInfo *files_info;
     PureFileInfo *file_info;
     size_t files_info_size;
@@ -586,7 +566,7 @@ static PureFileInfo *sreaddir(char **names_pnt)
     size_t names_counter = (size_t) 0U;
     size_t name_len;
     int (*cmp_func)(const void * const, const void * const);
-    
+
     if ((d = opendir(".")) == NULL) {
         return NULL;
     }
@@ -605,10 +585,10 @@ static PureFileInfo *sreaddir(char **names_pnt)
         if (checkprintable(de->d_name) != 0 || lstat(de->d_name, &st) < 0) {
             continue;
         }
-        name_len = strlen(de->d_name) + (size_t) 1U;        
+        name_len = strlen(de->d_name) + (size_t) 1U;
         while (names_counter + name_len >= names_size) {
             char *new_names;
-        
+
             if (name_len >= CHUNK_SIZE) {
                 names_size += name_len + CHUNK_SIZE;
             } else {
@@ -625,14 +605,14 @@ static PureFileInfo *sreaddir(char **names_pnt)
         }
         while ((files_info_counter + (size_t) 1U) >= files_info_size) {
             PureFileInfo *new_files_info;
-            
+
             files_info_size += (CHUNK_SIZE / sizeof *files_info);
-            if ((new_files_info = realloc(files_info, 
+            if ((new_files_info = realloc(files_info,
                                           files_info_size * sizeof *files_info)) == NULL) {
                 goto nomem;
             }
             files_info = new_files_info;
-        }        
+        }
         memcpy(&names[names_counter], de->d_name, name_len);   /* safe */
         names[names_counter + name_len - 1] = 0;
         file_info = &files_info[files_info_counter];
@@ -642,14 +622,14 @@ static PureFileInfo *sreaddir(char **names_pnt)
         file_info->mtime = st.st_mtime;
         file_info->mode = st.st_mode;
         file_info->nlink = st.st_nlink;
-        file_info->uid = st.st_uid;        
+        file_info->uid = st.st_uid;
         file_info->gid = st.st_gid;
         names_counter += name_len;
         files_info_counter++;
-    }    
+    }
     closedir(d);
     files_info[files_info_counter].name_offset = (size_t) -1;
-    *names_pnt = names;    
+    *names_pnt = names;
 
     if (opt_t) {
         if (opt_r) {
@@ -668,8 +648,8 @@ static PureFileInfo *sreaddir(char **names_pnt)
     } else {
         cmp_func = cmp;
     }
-    qsort(files_info, files_info_counter, sizeof files_info[0], cmp_func);    
-    
+    qsort(files_info, files_info_counter, sizeof files_info[0], cmp_func);
+
     return files_info;
 }
 
@@ -681,9 +661,10 @@ static void listdir(unsigned int depth, int f, void * const tls_fd,
     char *names;
     PureFileInfo *s;
     PureFileInfo *r;
-    char *c_buf;
+    char *alloca_subdir;
+    size_t sizeof_subdir;
     int d;
-    
+
     if (depth >= max_ls_depth || matches >= max_ls_files) {
         return;
     }
@@ -693,16 +674,16 @@ static void listdir(unsigned int depth, int f, void * const tls_fd,
     }
     s = dir;
     while (s->name_offset != (size_t) -1) {
+        d = 0;
         if (FI_NAME(s)[0] != '.') {
             d = listfile(s, NULL);
         } else if (opt_a) {
-            d = listfile(s, NULL);
             if (FI_NAME(s)[1] == 0 ||
                 (FI_NAME(s)[1] == '.' && FI_NAME(s)[2] == 0)) {
-                d = 0;
+                listfile(s, NULL);
+            } else {
+                d = listfile(s, NULL);
             }
-        } else {
-            d = 0;
         }
         if (!d) {
             s->name_offset = (size_t) -1;
@@ -711,84 +692,83 @@ static void listdir(unsigned int depth, int f, void * const tls_fd,
     }
     outputfiles(f, tls_fd);
     r = dir;
+    sizeof_subdir = PATH_MAX + 1U;
+    if ((alloca_subdir = ALLOCA(sizeof_subdir)) == NULL) {
+        goto toomany;
+    }
     while (opt_R && r != s) {
         if (r->name_offset != (size_t) -1 && !chdir(FI_NAME(r))) {
-            char *alloca_subdir;
-            const size_t sizeof_subdir = MAXPATHLEN + 1U;
-            
-            if ((alloca_subdir = ALLOCA(sizeof_subdir)) == NULL) {
-                goto toomany;
-            }
             if (SNCHECK(snprintf(alloca_subdir, sizeof_subdir, "%s/%s",
                                  name, FI_NAME(r)), sizeof_subdir)) {
                 goto nolist;
             }
-#ifdef WITH_RFC2640
-            c_buf = charset_fs2client(alloca_subdir);
-#else
-            c_buf = alloca_subdir;
-#endif
-#ifndef MINIMAL
-            if (modern_listings == 0) {
-#endif                
-#ifdef FANCY_LS_DIRECTORY_HEADERS
-                wrstr(f, tls_fd, "\r\n>----------------[");
-                wrstr(f, tls_fd, c_buf);
-                wrstr(f, tls_fd, "]----------------<\r\n\r\n");
-#else
-                wrstr(f, tls_fd, "\r\n\r\n");
-                wrstr(f, tls_fd, c_buf);
-                wrstr(f, tls_fd, ":\r\n\r\n");                    
-#endif
-#ifndef MINIMAL
-            }
-#endif
-#ifdef WITH_RFC2640            
-            free(c_buf);
-#endif            
+            wrstr(f, tls_fd, "\r\n\r\n");
+            wrstr(f, tls_fd, alloca_subdir);
+            wrstr(f, tls_fd, ":\r\n\r\n");
             listdir(depth + 1U, f, tls_fd, alloca_subdir);
+
             nolist:
-            ALLOCA_FREE(alloca_subdir);
             if (matches >= max_ls_files) {
                 goto toomany;
-            }                
+            }
             if (chdir("..")) {    /* defensive in the extreme... */
-                chdir(wd);
-                if (chdir(name)) {    /* someone rmdir()'d it? */
-                    die(421, LOG_ERR, "chdir: %s" ,
-                        strerror(errno));
+                if (chdir(wd) || chdir(name)) {    /* someone rmdir()'d it? */
+                    die(421, LOG_ERR, "chdir: %s", strerror(errno));
                 }
             }
         }
         r++;
     }
     toomany:
+    ALLOCA_FREE(alloca_subdir);
     free(names);
     free(dir);
     names = NULL;
 }
 
-void donlist(char *arg, const int on_ctrl_conn, const int opt_l_,
-             const int opt_a_, const int split_args)
+static char *unescape_and_return_next_file(char * const str) {
+    char *pnt = str;
+    signed char seen_backslash = 0;
+
+    while (*pnt != 0) {
+        if (seen_backslash == 0) {
+            if (*pnt == '\\') {
+                seen_backslash = 1;
+            } else if (*pnt == ' ') {
+                *pnt++ = 0;
+                if (*pnt != 0) {
+                    return pnt;
+                }
+                break;
+            }
+            pnt++;
+        } else {
+            seen_backslash = 0;
+            if (*pnt == ' ' || *pnt == '\\' || *pnt == '{' || *pnt == '}') {
+                memmove(pnt - 1, pnt, strlen(pnt) + (size_t) 1U);
+            }
+        }
+    }
+    return NULL;
+}
+
+void dolist(char *arg, const int on_ctrl_conn)
 {
     int c;
     void *tls_fd = NULL;
-    char *c_buf;
-    
+
     matches = 0U;
 
-    opt_C = opt_d = opt_F = opt_R = opt_r = opt_t = opt_S = 0;
-    opt_l = opt_l_;
+    opt_a = opt_C = opt_d = opt_F = opt_R = opt_r = opt_t = opt_S = 0;
+    opt_l = 1;
     if (force_ls_a != 0) {
         opt_a = 1;
-    } else {
-        opt_a = opt_a_;
     }
-    if (split_args != 0) {
+    if (arg != NULL) {
         while (isspace((unsigned char) *arg)) {
             arg++;
         }
-        while (arg && *arg == '-') {
+        while (*arg == '-') {
             while (arg++ && isalnum((unsigned char) *arg)) {
                 switch (*arg) {
                 case 'a':
@@ -838,18 +818,16 @@ void donlist(char *arg, const int on_ctrl_conn, const int opt_l_,
             return;
         }
         doreply();
-        if (type == 2) {
-            addreply_noformat(0, "ASCII");
-        }
 #ifdef WITH_TLS
         if (data_protection_level == CPL_PRIVATE) {
+            tls_init_data_session(xferfd, passive);
             tls_fd = tls_data_cnx;
         }
 #endif
     } else {                           /* STAT command */
         c = clientfd;
 #ifdef WITH_TLS
-        if (data_protection_level == CPL_PRIVATE) {
+        if (tls_cnx != NULL) {
             secure_safe_write(tls_cnx, "213-STAT" CRLF,
                               sizeof "213-STAT" CRLF - 1U);
             tls_fd = tls_cnx;
@@ -857,7 +835,7 @@ void donlist(char *arg, const int on_ctrl_conn, const int opt_l_,
         else
 #endif
         {
-            safe_write(c, "213-STAT" CRLF, sizeof "213-STAT" CRLF - 1U);
+            safe_write(c, "213-STAT" CRLF, sizeof "213-STAT" CRLF - 1U, -1);
         }
     }
     if (arg != NULL && *arg != 0) {
@@ -870,33 +848,30 @@ void donlist(char *arg, const int on_ctrl_conn, const int opt_l_,
             int a;
             char *endarg;
 
-            if (split_args == 0) {
-                endarg = NULL;
-            } else if ((endarg = strchr(arg, ' ')) != NULL) {
-                *endarg++ = 0;
+            if ((endarg = unescape_and_return_next_file(arg)) != NULL) {
                 justone = 0;
             }
-#ifdef DEBUG
-            if (debug != 0) {
-                addreply(226, "Glob: %s", arg);
-            }
-#endif
 
             /* Expand ~ here if needed */
-            
+
             alarm(GLOB_TIMEOUT);
+            memset(&g, 0, sizeof g);
             a = sglob(arg,
-                      GLOB_BRACE | (opt_a ? GLOB_PERIOD : 0),
-                     NULL, &g, max_ls_files + 2, max_ls_depth * 2);
+                      opt_a ? (GLOB_PERIOD | GLOB_LIMIT) : GLOB_LIMIT,
+                      NULL, &g, max_ls_files + 2, max_ls_depth * 2);
             alarm(0);
             if (a == 0) {
                 char **path;
 
-                path = g.gl_pathv;
-                if (path && path[0] && path[1]) {
+                if (g.gl_pathc <= 0) {
+                    path = NULL;
+                } else {
+                    path = g.gl_pathv;
+                }
+                if (path != NULL && path[0] != NULL && path[1] != NULL) {
                     justone = 0;
                 }
-                while (path && *path) {
+                while (path != NULL && *path != NULL) {
                     struct stat st;
 
                     if (stat(*path, &st) == 0) {
@@ -911,33 +886,21 @@ void donlist(char *arg, const int on_ctrl_conn, const int opt_l_,
                 }
                 outputfiles(c, tls_fd);    /* in case of opt_C */
                 path = g.gl_pathv;
-                while (path && *path) {
+                while (path != NULL && *path != NULL) {
                     if (matches >= max_ls_files) {
                         break;
                     }
-                    if (**path) {
+                    if (**path != 0) {
                         if (!justone) {
-#ifdef WITH_RFC2640                            
-                            c_buf = charset_fs2client(*path);
-#else
-                            c_buf = *path;
-#endif
-#ifdef FANCY_LS_DIRECTORY_HEADERS                            
-                            wrstr(c, tls_fd, "\r\n>-----------------[");
-                            wrstr(c, tls_fd, c_buf);
-                            wrstr(c, tls_fd, "]-----------------<\r\n\r\n");
-#else
                             wrstr(c, tls_fd, "\r\n\r\n");
-                            wrstr(c, tls_fd, c_buf);
+                            wrstr(c, tls_fd, *path);
                             wrstr(c, tls_fd, ":\r\n\r\n");
-#endif
-#ifdef WITH_RFC2640                            
-                            free(c_buf);
-#endif
                         }
                         if (!chdir(*path)) {
                             listdir(0U, c, tls_fd, *path);
-                            chdir(wd);
+                            if (chdir(wd)) {
+                                die(421, LOG_ERR, "chdir: %s", strerror(errno));
+                            }
                         }
                     }
                     path++;
@@ -968,7 +931,7 @@ void donlist(char *arg, const int on_ctrl_conn, const int opt_l_,
     if (on_ctrl_conn == 0) {
 #ifdef WITH_TLS
         closedata();
-#endif    
+#endif
         close(c);
     } else {
         addreply_noformat(213, "End.");
@@ -990,6 +953,153 @@ void donlist(char *arg, const int on_ctrl_conn, const int opt_l_,
     } else {
         addreply(226, MSG_LS_SUCCESS, matches);
     }
-    end:
-    chdir(wd);
+end:
+    if (chdir(wd)) {
+        die(421, LOG_ERR, "chdir: %s", strerror(errno));
+    }
+}
+
+void donlst(const char *base)
+{
+    char           line[PATH_MAX + 3U];
+    DIR           *dir;
+    void          *tls_fd = NULL;
+    struct dirent *de;
+    size_t         name_len;
+    unsigned int   matches = 0;
+    int            c;
+    int            base_has_trailing_slash = 0;
+
+    if (*base != 0 && chdir(base) != 0) {
+        if (*base++ == '-') {
+            while (!isspace((unsigned char) *base++));
+            while (isspace((unsigned char) *base++));
+            if (*base != 0 && chdir(base) != 0) {
+                addreply_noformat(550, MSG_STAT_FAILURE2);
+                return;
+            }
+        } else {
+            addreply_noformat(550, MSG_STAT_FAILURE2);
+            return;
+        }
+    }
+    if (*base !=0 && base[strlen(base) - 1U] == '/') {
+        base_has_trailing_slash = 1;
+    }
+    if ((dir = opendir(".")) == NULL) {
+        addreply_noformat(550, MSG_STAT_FAILURE2);
+        goto bye;
+    }
+    opendata();
+    if ((c = xferfd) == -1) {
+        goto bye;
+    }
+    doreply();
+#ifdef WITH_TLS
+    if (data_protection_level == CPL_PRIVATE) {
+        tls_init_data_session(xferfd, passive);
+        tls_fd = tls_data_cnx;
+    }
+#endif
+    while ((de = readdir(dir)) != NULL) {
+        if (checkprintable(de->d_name) != 0) {
+            continue;
+        }
+        name_len = strlen(de->d_name);
+        if (name_len > sizeof line - 3U) {
+            continue;
+        }
+        memcpy(line, de->d_name, name_len);
+        line[name_len] = '\r';
+        line[name_len + 1] = '\n';
+        line[name_len + 2] = 0;
+        if (*base) {
+            wrstr(c, tls_fd, base);
+            if (base_has_trailing_slash == 0) {
+                wrstr(c, tls_fd, "/");
+            }
+        }
+        wrstr(c, tls_fd, line);
+        matches++;
+        if (matches >= max_ls_files) {
+            break;
+        }
+    }
+    closedir(dir);
+    wrstr(c, tls_fd, NULL);
+    closedata();
+    if (matches >= max_ls_files) {
+        addreply(226, MSG_LS_TRUNCATED, matches);
+    } else {
+        addreply(226, MSG_LS_SUCCESS, matches);
+    }
+bye:
+    if (chdir(wd) != 0) {
+        die(421, LOG_ERR, "chdir: %s", strerror(errno));
+    }
+}
+
+void domlsd(const char *base)
+{
+    char           line[PATH_MAX + 1];
+    DIR           *dir = NULL;
+    void          *tls_fd = NULL;
+    struct dirent *de;
+    unsigned int   matches = 0;
+    int            c;
+
+    if (*base != 0 && chdir(base) != 0) {
+        if (*base++ == '-') {
+            while (!isspace((unsigned char) *base++));
+            while (isspace((unsigned char) *base++));
+            if (*base != 0 && chdir(base) != 0) {
+                addreply_noformat(550, MSG_STAT_FAILURE2);
+                return;
+            }
+        } else {
+            addreply_noformat(550, MSG_STAT_FAILURE2);
+            return;
+        }
+    }
+    if ((dir = opendir(".")) == NULL) {
+        addreply_noformat(550, MSG_STAT_FAILURE2);
+        goto bye;
+    }
+    opendata();
+    if ((c = xferfd) == -1) {
+        goto bye;
+    }
+    doreply();
+#ifdef WITH_TLS
+    if (data_protection_level == CPL_PRIVATE) {
+        tls_init_data_session(xferfd, passive);
+        tls_fd = tls_data_cnx;
+    }
+#endif
+    while ((de = readdir(dir)) != NULL) {
+        if (checkprintable(de->d_name) != 0 ||
+            modernformat(de->d_name, line, sizeof line, "") < 0) {
+            continue;
+        }
+        wrstr(c, tls_fd, line);
+        wrstr(c, tls_fd, "\r\n");
+        matches++;
+        if (matches >= max_ls_files) {
+            break;
+        }
+    }
+    wrstr(c, tls_fd, NULL);
+    closedata();
+    if (matches >= max_ls_files) {
+        addreply(226, MSG_LS_TRUNCATED, matches);
+    } else {
+        addreply(226, MSG_LS_SUCCESS, matches);
+    }
+bye:
+    if (dir != NULL) {
+        closedir(dir);
+    }
+    if (chdir(wd) != 0) {
+        die(421, LOG_ERR, "chdir: %s", strerror(errno));
+    }
 }
