@@ -1,7 +1,11 @@
 #ifndef __MESSAGES_H__
 #define __MESSAGES_H__ 1
 
-#ifdef MESSAGES_DE
+#if defined(MESSAGES_SQ)
+# define MESSAGES_LOCALE "sq_AL"
+# include "messages_sq.h"
+
+#elif defined(MESSAGES_DE)
 # define MESSAGES_LOCALE "de_DE"
 # include "messages_de.h"
 
@@ -73,6 +77,18 @@
 # define MESSAGES_LOCALE "cs_CZ"
 # include "messages_cs_cz.h"
 
+#elif defined(MESSAGES_TR)
+# define MESSAGES_LOCALE "tr_TR"
+# include "messages_tr.h"
+
+#elif defined(MESSAGES_HU)
+# define MESSAGES_LOCALE "hu_HU"
+# include "messages_hu.h"
+
+#elif defined(MESSAGES_CA_ES)
+# define MESSAGES_LOCALE "ca_ES"
+# include "messages_ca_es.h"
+
 #else
 # define MESSAGES_LOCALE "en_GB"
 # include "messages_en.h"
