@@ -1,3 +1,4 @@
+#define MSG_TLS_CIPHER_FAILED "TLS: Invalid TLSCipherSuite specified '%s'"
 #define MSG_TIMEOUT "as vyprel"
 #define MSG_CAPABILITIES "Nemohu zmnit privilegia"
 #define MSG_CLIENT_CLOSED_CNX "Klient uzavel spojen"
@@ -87,7 +88,7 @@
 #define MSG_ANON_CANT_DELETE "Anonymn uivatel neme mazat soubory"
 #define MSG_ANON_CANT_OVERWRITE "Anonymn uivatel neme pepisovat existujc soubory"
 #define MSG_DELE_FAILED "%s nelze smazat"
-#define MSG_DELE_SUCCESS "Soubor %s smazn"
+#define MSG_DELE_SUCCESS "Soubor %s%s%s%s smazn"
 #define MSG_DELE_TOTAL_FAILURE "dn soubor nebyl smazn"
 #define MSG_LOAD_TOO_HIGH \
   "Zt systmu, kdy jste se pihlsil, byla %3.2f. Nedovolujeme dowloady\n" \
@@ -172,7 +173,7 @@
 #define MSG_NEW_CONNECTION "Nov spojen z %s"
 #define MSG_WELCOME_TO "Vtejte na"
 #define MSG_MAX_USERS "Lituji, ale %lu (maximum) uivatel je ji pihlench"
-#define MSG_NB_USERS "Jste uivatelem slo %d z %d povolench"
+#define MSG_NB_USERS "Jste uivatelem slo %u z %u povolench"
 #define MSG_WELCOME_TIME "Lokaln as je %02d:%02d. Port serveru je %u."
 #define MSG_ANONYMOUS_FTP_ONLY "Zde je povolen pouze anonymn pstup"
 #define MSG_RATIOS_EVERYONE "VICHNI UIVATELE MUSI DODRET POMER UL/DL :"
@@ -205,7 +206,7 @@
 #define MSG_ILLEGAL_QUOTA "Chybn kvota"
 #define MSG_QUOTA_FILES "%llu soubor pouitch (%d%%) - autorizovanch: %llu soubor"
 #define MSG_QUOTA_SIZE "%llu kilobajt pouitch (%d%%) - autorizovanch: %llu kilobajt"
-#define MSG_QUOTA_EXCEEDED "Kvota pekroena: %s nebude uloen"
+#define MSG_QUOTA_EXCEEDED "Kvota pekroena: [%s] nebude uloen"
 #define MSG_AUTH_UNKNOWN "Neznm autentifikan metoda"
 #define MSG_PDB_BROKEN "Nebylo mon pest indexovan soubor puredb (nebo byla detekovna star verze) - Vyzkouejte spustit pure-pw mkdb"
 #define MSG_ALIASES_ALIAS "%s je alias pro %s."
@@ -214,3 +215,19 @@
 #define MSG_ALIASES_LIST "Jsou dostupn nasledujc aliasy:"
 #define MSG_PERUSER_MAX "Nemohu akceptovat vc ne %lu pipojen stejnho uivatele"
 #define MSG_IPV6_OK "Na tento server se lze rovn pipojovat pouitm IPv6 protokolu"
+#define MSG_TLS_INFO "TLS: Enabled %s with %s, %d secret bits cipher"
+#define MSG_TLS_WEAK "TLS: Cipher too weak"
+#define MSG_TLS_NEEDED "Sorry, cleartext sessions are not accepted on this server.\n" \
+    "Please reconnect using TLS security mechanisms."
+#define MSG_ILLEGAL_CHARSET "Illegal charset"
+#define MSG_TLS_NO_CTX "TLS: Context not found. Exiting."
+#define MSG_PROT_OK "Data protection level set to \"%s\""
+#define MSG_PROT_PRIVATE_NEEDED "Data connection cannot be opened with this PROT setting."
+#define MSG_PROT_UNKNOWN_LEVEL "Protection level %s not understood. Fallback to \"%s\""
+#define MSG_PROT_BEFORE_PBSZ "PROT must be preceded by a successful PBSZ command"
+#define MSG_WARN_LDAP_USERPASS_EMPTY "LDAP returned no userPassword attribute, check LDAP access rights."
+#define MSG_LDAP_INVALID_AUTH_METHOD "Invalid LDAPAuthMethod in the configuration file. Should be 'bind' or 'password'."
+#define MSG_INVALID_ARGUMENT "Neplatn argument: \"%s\""
+#define MSG_READY_TO_PROCEED "Ready to proceed"
+#define MSG_ASCII_MODE_UNSUPPORTED "Transfers in ASCII mode are not supported"
+#define MSG_ASCII_MODE_WARNING "Warning: client is in ASCII mode - Assuming data is plain text"
