@@ -1,4 +1,4 @@
-#define MSG_TLS_CIPHER_FAILED "SSL/TLS: Fehlerhafte TLSCipherSuite definiert '%s'"
+#define MSG_TLS_CIPHER_FAILED "TLS: Fehlerhafte TLSCipherSuite definiert '%s'"
 #define MSG_TIMEOUT "Zeitüberschreitung"
 #define MSG_CAPABILITIES "Berechtigungen (CAPABILITES) konnten nicht gewechselt werden."
 #define MSG_CLIENT_CLOSED_CNX "Die Gegenstelle hat die Verbindung geschlossen."
@@ -7,13 +7,13 @@
 #define MSG_CANT_CREATE_DATA_SOCKET "Datenverbindung kann nicht aufgebaut werden."
 #define MSG_DEBUG_CLIENT_IS "Addresse der Gegenstelle: "
 #define MSG_SYNTAX_ERROR_IP "Syntaxfehler in IP-Addresse."
-#define MSG_PORT_SUCCESSFUL "PORT-Kommando erfolgreich."
-#define MSG_ONLY_IPV4V6 "Es werden nur IPv4 und IPv6 unterstätzt (1,2)."
+#define MSG_PORT_SUCCESSFUL "PORT-Befehl erfolgreich."
+#define MSG_ONLY_IPV4V6 "Es werden nur IPv4 und IPv6 unterstützt (1,2)."
 #define MSG_ONLY_IPV4 "Es wird nur IPv4 unterstützt (1)."
 #define MSG_TIMEOUT_PARSER "Zeitüberschreitung."
 #define MSG_LINE_TOO_LONG "Die Zeile ist zu lang."
-#define MSG_LOG_OVERFLOW "Die Gegenstelle hat versucht, den Kommandozeilenbuffer zu überfluten."
-#define MSG_GOODBYE "Tschüs. Verbindungen: Upload %llu und Download %llu kbytes."
+#define MSG_LOG_OVERFLOW "Die Gegenstelle hat versucht den Kommandozeilenbuffer zu überfluten."
+#define MSG_GOODBYE "Tschüss. Verbindungen: Upload %llu und Download %llu kbytes."
 #define MSG_DEBUG_COMMAND "Befehl"
 #define MSG_IS_YOUR_CURRENT_LOCATION "ist das aktuelle Verzeichnis."
 #define MSG_NOT_LOGGED_IN "Sie sind nicht angemeldet."
@@ -29,7 +29,7 @@
 #define MSG_SITE_HELP "Die folgenden SITE-Befehle werden unterstützt"
 #define MSG_BAD_CHMOD "Ungültige Rechte."
 #define MSG_UNKNOWN_EXTENSION "ist eine unbekannte Erweiterung."
-#define MSG_XDBG_OK "XDBG-Befehl erfolgreich, debug-level ist jetzt %d."
+#define MSG_XDBG_OK "XDBG-Befehl erfolgreich, Debug-Level ist jetzt %d."
 #define MSG_UNKNOWN_COMMAND "Unbekannter Befehl."
 #define MSG_TIMEOUT_NOOP "Zeitüberschreitung (keine Aktion seit %lu Sekunden)."
 #define MSG_TIMEOUT_DATA "Zeitüberschreitung (keine neuen Daten seit %lu Sekunden)."
@@ -38,13 +38,13 @@
 #define MSG_ANY_PASSWORD "Jedes Passwort wird akzeptiert."
 #define MSG_ANONYMOUS_LOGGED "Anonymer Benutzer angemeldet."
 #define MSG_ANONYMOUS_LOGGED_VIRTUAL "Anonymer Benutzer im virtuellen FTP eingeloggt."
-#define MSG_USER_OK "Benutzer %s GEFUNDEN. Passwort erforderlich."
+#define MSG_USER_OK "Benutzer %s gefunden. Passwort erforderlich."
 #define MSG_CANT_DO_TWICE "Aktion kann nicht durchgeführt werden."
 #define MSG_UNABLE_SECURE_ANON "Sicheres anonymes FTP konnte nicht bereitgestellt werden."
-#define MSG_BANDWIDTH_RESTRICTED "Ihre Bandbreitenbenutzung ist limitiert."
+#define MSG_BANDWIDTH_RESTRICTED "Die Bandbreite ist limitiert."
 #define MSG_NO_PASSWORD_NEEDED "Wozu ein Passwort?"
 #define MSG_NOTRUST "Kann ich Ihnen vertrauen?"
-#define MSG_WHOAREYOU "Wer sind Sie ?"
+#define MSG_WHOAREYOU "Wer sind Sie?"
 #define MSG_AUTH_FAILED "Authentifizierung fehlgeschlagen."
 #define MSG_AUTH_TOOMANY "Zu viele Authentifizierungsfehler."
 #define MSG_NO_HOMEDIR "Home-Verzeichnis nicht erreichbar - Abbruch."
@@ -55,27 +55,27 @@
 #define MSG_RATIO "Sie müssen ein %u:%u (UL/DL) - Verhältnis akzeptieren."
 #define MSG_CHROOT_FAILED "Konnte kein sicheres chroot()-Gefängnis einrichten."
 #define MSG_CURRENT_DIR_IS "Aktuelles Verzeichnis ist %s."
-#define MSG_CURRENT_RESTRICTED_DIR_IS "Aktuelles, beschränktes Verzeichnis ist %s."
+#define MSG_CURRENT_RESTRICTED_DIR_IS "Aktuelles beschränktes Verzeichnis ist %s."
 #define MSG_IS_NOW_LOGGED_IN "%s ist jetzt eingeloggt."
 #define MSG_CANT_CHANGE_DIR "Verzeichnis %s nicht erreichbar"
 #define MSG_PATH_TOO_LONG "Pfad ist zu lang."
 #define MSG_CANT_PASV "PASV ist nicht geeignet für IPv6. Nehmen Sie EPSV."
 #define MSG_CANT_PASSIVE "Kann keine passive Verbindung herstellen."
-#define MSG_PORTS_BUSY "Alle reservierten TCP-ports sind belegt."
+#define MSG_PORTS_BUSY "Alle reservierten TCP-Ports sind belegt."
 #define MSG_GETSOCKNAME_DATA "Kann den lokalen Daten-Socket nicht identifizieren."
 #define MSG_GETPEERNAME "Kann den lokalen Socket nicht identifizieren."
 #define MSG_INVALID_IP "Ungültige Adresse."
 #define MSG_NO_EPSV "Bitte benutzen Sie einen IPv6-konformen Klienten mit EPSV-Unterstützung."
-#define MSG_BAD_PORT "Ich verbinde nicht zu ports kleiner als 1024."
+#define MSG_BAD_PORT "Ich verbinde nicht zu Ports kleiner als 1024."
 #define MSG_NO_FXP "Ich werde keine Verbindung mit %s herstellen (nur zu %s)."
-#define MSG_FXP "FXP-Transfer: von %s to %s."
+#define MSG_FXP "FXP-Transfer: von %s nach %s."
 #define MSG_NO_DATA_CONN "Keine Datenverbindung."
 #define MSG_ACCEPT_FAILED "Die Verbindung wird nicht akzeptiert."
 #define MSG_ACCEPT_SUCCESS "Verbindung akzeptiert."
 #define MSG_CNX_PORT_FAILED "Kann keine Datenverbindung zu Port %d aufbauen."
 #define MSG_CNX_PORT "Verbinde mit Port %d."
-#define MSG_ANON_CANT_MKD "Sorry, anonyme Benutzer dürfen keine Verzeichnisse anlegen."
-#define MSG_ANON_CANT_RMD "Sorry, anonyme Benutzer dürfen keine Verzeichnisse löschen."
+#define MSG_ANON_CANT_MKD "Anonyme Benutzer dürfen keine Verzeichnisse anlegen."
+#define MSG_ANON_CANT_RMD "Anonyme Benutzer dürfen keine Verzeichnisse löschen."
 #define MSG_ANON_CANT_RENAME "Anonyme Benutzer dürfen keine Dateien verschieben oder umbenennen."
 #define MSG_ANON_CANT_CHANGE_PERMS "Anonyme Benutzer dürfen die Rechte nicht veränderen."
 #define MSG_GLOB_NO_MEMORY "Nicht genug Speicher beim Vervollständigen von %s."
@@ -92,8 +92,7 @@
 #define MSG_DELE_TOTAL_FAILURE "Keine Datei gelöscht."
 #define MSG_LOAD_TOO_HIGH \
     "Zum Zeitpunkt der Verbindung war die Last %3.2f. Wir erlauben keinen Download\n" \
-    "durch anonyme Benutzer wenn die Last so hoch ist. Uploads sind\n" \
-    "immer erlaubt."
+    "durch anonyme Benutzer, wenn die Last so hoch ist. Uploads sind immer erlaubt."
 #define MSG_OPEN_FAILURE "Kann %s nicht öffnen"
 #define MSG_OPEN_FAILURE2 "Kann diese Datei nicht öffnen."
 #define MSG_STAT_FAILURE "Kann Dateigröße nicht ermitteln."
@@ -112,8 +111,8 @@
 #define MSG_WINNER "Der Computer ist Ihr Freund. Trauen Sie ihrem Computer."
 #define MSG_KBYTES_LEFT "noch %.1f kbytes."
 #define MSG_ABORTED "Transfer abgebrochen."
-#define MSG_DATA_WRITE_FAILED "Fehler während Schreibvorgang zur Datenverbindung."
-#define MSG_DATA_READ_FAILED "Fehler während Lesevorgang von der Datenverbindung."
+#define MSG_DATA_WRITE_FAILED "Fehler während des Schreibvorgangs zur Datenverbindung."
+#define MSG_DATA_READ_FAILED "Fehler während des Lesevorgangs von der Datenverbindung."
 #define MSG_MMAP_FAILED "Kann die Datei nicht im Speicher halten."
 #define MSG_WRITE_FAILED "Fehler beim Abspeichern als Datei."
 #define MSG_TRANSFER_RATE_M "%.3f s (hier gemessen), %.2f Mbytes pro Sekunde."
@@ -122,7 +121,7 @@
 #define MSG_SPACE_FREE_M "%.1f Mbytes freier Festplattenplatz."
 #define MSG_SPACE_FREE_K "%f Kbytes freier Festplattenplatz."
 #define MSG_DOWNLOADED "heruntergeladen."
-#define MSG_REST_NOT_NUMERIC "REST benötigt einen numerischen Paramter."
+#define MSG_REST_NOT_NUMERIC "REST benötigt einen numerischen Parameter."
 #define MSG_REST_ASCII_STRICT "Antwort-Markierung muss im ASCII-Modus 0 sein."
 #define MSG_REST_ASCII_WORKAROUND "Neustart bei %lld. Aber im ASCII-Modus."
 #define MSG_REST_SUCCESS "Neustart bei %lld."
@@ -153,11 +152,11 @@
 #define MSG_RENAME_FAILURE "Fehler beim Umbenennen/Verschieben."
 #define MSG_RENAME_SUCCESS "Datei erfolgreich umbenannt oder verschoben."
 #define MSG_NO_SUPERSERVER "Bitte lassen Sie pure-ftpd von einem super-server (tcpserver, xinetd, ...) starten."
-#define MSG_NO_FTP_ACCOUNT "Kann das 'ftp' - Nutzerkonto nicht finden."
+#define MSG_NO_FTP_ACCOUNT "Kann das 'ftp' - Benutzerkonto nicht finden."
 #define MSG_CONF_ERR "Konfigurationsfehler."
 #define MSG_NO_VIRTUAL_FILE "Datei für virtuelle Benutzer fehlt."
 #define MSG_ILLEGAL_THROTTLING "Ungültiger Wert für die Bandbreitenbegrenzung."
-#define MSG_ILLEGAL_TRUSTED_GID "Ungültige Gruppenid für chroot."
+#define MSG_ILLEGAL_TRUSTED_GID "Ungültige Gruppen-Id für chroot()-Gefängnis."
 #define MSG_ILLEGAL_USER_LIMIT "Ungültige Benutzergrenze."
 #define MSG_ILLEGAL_FACILITY "Name der facility unbekannt."
 #define MSG_ILLEGAL_CONFIG_FILE_LDAP "Ungültige LDAP-Konfigurationsdatei."
@@ -165,7 +164,7 @@
 #define MSG_ILLEGAL_PORTS_RANGE "Ungültiger Portbereich."
 #define MSG_ILLEGAL_LS_LIMITS "Ungültige Beschränkungen für 'ls'."
 #define MSG_ILLEGAL_FORCE_PASSIVE "Die erzwungene IP-Adresse für passive Verbindungen ist ungültig."
-#define MSG_ILLEGAL_RATIO "Ungültige Upload/Download - Beschränkung."
+#define MSG_ILLEGAL_RATIO "Ungültige Upload/Download-Beschränkung."
 #define MSG_ILLEGAL_UID_LIMIT "Ungültige uid-Beschränkung."
 #define MSG_ILLEGAL_OPTION "Unbekannte Laufzeitoption."
 #define MSG_LDAP_MISSING_BASE "Fehlendes LDAP-BaseDN in der LDAP-Konfigurationsdatei."
@@ -176,8 +175,8 @@
 #define MSG_NB_USERS "Sie sind Benutzer %u von %u erlaubten."
 #define MSG_WELCOME_TIME "Lokale Zeit: %02d:%02d. Serverport: %u."
 #define MSG_ANONYMOUS_FTP_ONLY "Nur anonymes FTP erlaubt."
-#define MSG_RATIOS_EVERYONE "RATIOS SIND FÜR JEDEN AKTIVIERT:"
-#define MSG_RATIOS_ANONYMOUS "ANONYME NUTZER UNTERLIEGEN EINER UPLOAD/DOWNLOAD-QUOTE:"
+#define MSG_RATIOS_EVERYONE "Es besteht eine Upload/Download-Beschränkung:"
+#define MSG_RATIOS_ANONYMOUS "Anonyme Benutzer unterliegen einer Upload/Download-Beschränkung:"
 #define MSG_RATIOS_RULE "um %u Mb herunterzuladen, müssen %u Mb hochgeladen werden."
 #define MSG_INFO_IDLE_M "Sie werden nach %lu Minuten Inaktivität getrennt."
 #define MSG_INFO_IDLE_S "Sie werden nach %lu Sekunden Inaktivität getrennt."
@@ -188,12 +187,12 @@
 #define MSG_AUTH_FAILED_LOG "Authentifizierung fehlgeschlagen für Benutzer [%s]."
 #define MSG_ILLEGAL_UMASK "Ungültige umask."
 #define MSG_STANDALONE_FAILED "Kann keinen autarken Server starten"
-#define MSG_NO_ANONYMOUS_LOGIN "Dies ist ein privates System - Keine anonyme Anmeldung möglich."
-#define MSG_ANONYMOUS_ANY_PASSWORD "jedes Paßwort funktioniert."
+#define MSG_NO_ANONYMOUS_LOGIN "Dies ist ein privates System - keine anonyme Anmeldung möglich."
+#define MSG_ANONYMOUS_ANY_PASSWORD "jedes Passwort ist zulässig."
 #define MSG_MAX_USERS_IP "Zuviele Verbindungen (%lu) von dieser IP-Adresse."
 #define MSG_ACTIVE_DISABLED "Aktives FTP ist auf diesem Server verboten."
 #define MSG_TRANSFER_SUCCESSFUL "Datei erfolgreich übertragen."
-#define MSG_NO_DISK_SPACE "Festplatte voll - bitte spüter hochladen."
+#define MSG_NO_DISK_SPACE "Festplatte voll - bitte später erneut probieren."
 #define MSG_OUT_OF_MEMORY "Nicht genug Arbeitsspeicher."
 #define MSG_ILLEGAL_TRUSTED_IP "Ungültige vertraute IP-Adresse."
 #define MSG_NO_ASCII_RESUME "Wiederaufnahme im ASCII-Modus ist unsicher, bitte löschen Sie die Datei zuerst."
@@ -213,17 +212,21 @@
 #define MSG_ALIASES_UNKNOWN "Unbekannter Alias %s."
 #define MSG_ALIASES_BROKEN_FILE "Beschädigte Aliasdatei"
 #define MSG_ALIASES_LIST "Folgende Aliase stehen zur Verfügung:"
-#define MSG_PERUSER_MAX "Mehr als %lu Verbindungen vom selben Benutzer sind nicht erlaubt"
-#define MSG_IPV6_OK "Auf diesem Server sind auch IPv6-Verbindungen willkommen"
-#define MSG_TLS_INFO "SSL/TLS: %s Verschlüsselung mit %s, %d bit Schlüssellänge"
-#define MSG_TLS_WEAK "SSL/TLS: Verschlüsselung zu schwach"
-#define MSG_TLS_NEEDED "Sorry, unverschlüsselte Sitzungen werden auf diesem Server nicht akzeptiert.\n" \
-    "Bitte verbinden Sie sich mittels SSL/TLS-Verschlüsselung"
-#define MSG_ILLEGAL_CHARSET "Illegal charset"
-#define MSG_TLS_NO_CTX "SSL/TLS: Context not found. Exiting."
-#define MSG_PROT_OK "Data protection level set to \"%s\""
-#define MSG_PROT_PRIVATE_NEEDED "Data connection cannot be opened with this PROT setting."
-#define MSG_PROT_UNKNOWN_LEVEL "Protection level %s not understood. Fallback to \"%s\""
-#define MSG_PROT_BEFORE_PBSZ "PROT must be preceded by a successful PBSZ command"
-#define MSG_WARN_LDAP_USERPASS_EMPTY "LDAP returned no userPassword attribute, check LDAP access rights."
-#define MSG_LDAP_INVALID_AUTH_METHOD "Invalid LDAPAuthMethod in the configuration file. Should be 'bind' or 'password'."
+#define MSG_PERUSER_MAX "Mehr als %lu Verbindungen vom selben Benutzer sind nicht erlaubt."
+#define MSG_IPV6_OK "Auf diesem Server sind auch IPv6-Verbindungen willkommen."
+#define MSG_TLS_INFO "TLS: %s Verschlüsselung mit %s, %d bit Schlüssellänge"
+#define MSG_TLS_WEAK "TLS: Verschlüsselung zu schwach"
+#define MSG_TLS_NEEDED "Dieser Server akzeptiert keine unverschlüsselten Verbindungen.\n" \
+    "Bitte nutzen Sie eine TLS-Verschlüsselung zur Verbindung."
+#define MSG_ILLEGAL_CHARSET "Ungültiger Zeichensatz"
+#define MSG_TLS_NO_CTX "TLS: Verschlüsselungskontext nicht gefunden. Tschüss."
+#define MSG_PROT_OK "Datensicherungslevel ist \"%s\""
+#define MSG_PROT_PRIVATE_NEEDED "Datenverbindung kann mit diesen PROT Einstellungen nicht aufgebaut werden."
+#define MSG_PROT_UNKNOWN_LEVEL "Datensicherungslevel \"%s\" unbekannt. Nutze \"%s\" stattdessen."
+#define MSG_PROT_BEFORE_PBSZ "Vor einem PROT-Befehl muss ein PBSZ-Befehl erfolgt sein."
+#define MSG_WARN_LDAP_USERPASS_EMPTY "Der LDAP-Server hat kein 'userPassword'-Attribut zurückgeliefert - überprüfe die LDAP-Zugriffsrechte."
+#define MSG_LDAP_INVALID_AUTH_METHOD "Ungültiger Wert für LDAPAuthMethod in der Konfigurationsdatei gefunden - nur 'bind' oder 'password' sind erlaubt."
+#define MSG_INVALID_ARGUMENT "Ungültiger Wert: \"%s\""
+#define MSG_READY_TO_PROCEED "Bereit fortzufahren"
+#define MSG_ASCII_MODE_UNSUPPORTED "Übertragungen im ASCII-Modus werden nicht unterstützt."
+#define MSG_ASCII_MODE_WARNING "Warnung: Der Klient nutzt den ASCII-Modus. Es wird angenommen, dass es sich um einfache Text Dateien handelt."
