@@ -1,3 +1,4 @@
+#define MSG_TLS_CIPHER_FAILED "TLS: Invalid TLSCipherSuite specified '%s'"
 #define MSG_TIMEOUT "Timeout"
 #define MSG_CAPABILITIES "No se han podido alternar capacidades"
 #define MSG_CLIENT_CLOSED_CNX "El cliente cerr la conexin"
@@ -87,7 +88,7 @@
 #define MSG_ANON_CANT_DELETE "Los usuarios annimos no pueden borrar archivos"
 #define MSG_ANON_CANT_OVERWRITE "Los usuarios annimos no pueden sobreescribir archivos existentes"
 #define MSG_DELE_FAILED "No se pudo borrar %s"
-#define MSG_DELE_SUCCESS "Borrado %s"
+#define MSG_DELE_SUCCESS "Borrado %s%s%s%s"
 #define MSG_DELE_TOTAL_FAILURE "Archivo no borrado"
 #define MSG_LOAD_TOO_HIGH \
     "La carga era %3.2f cuando conectaste. No permitimos descargas\n" \
@@ -172,7 +173,7 @@
 #define MSG_NEW_CONNECTION "Nueva conexin desde %s"
 #define MSG_WELCOME_TO "Bienvenido a"
 #define MSG_MAX_USERS "%lu usuarios (el mximo) estn el sistema, lo siento"
-#define MSG_NB_USERS "Eres el usuario nmero %d de %d permitidos"
+#define MSG_NB_USERS "Eres el usuario nmero %u de %u permitidos"
 #define MSG_WELCOME_TIME "La hora local es ahora %02d:%02d. Puerto del servidor: %u."
 #define MSG_ANONYMOUS_FTP_ONLY "Slo se permite FTP annimo"
 #define MSG_RATIOS_EVERYONE "LAS PROPORCIONES ESTN ESTABLECIDAS PARA TODOS :"
@@ -205,7 +206,7 @@
 #define MSG_ILLEGAL_QUOTA "Cuota invlida"
 #define MSG_QUOTA_FILES "%llu archivos utilizados (%d%%) - autorizados: %llu archivos"
 #define MSG_QUOTA_SIZE "%llu Kbytes utilizados (%d%%) - autorizados: %llu Kb"
-#define MSG_QUOTA_EXCEEDED "Cuota excedida : %s no ser guardado"
+#define MSG_QUOTA_EXCEEDED "Cuota excedida : [%s] no ser guardado"
 #define MSG_AUTH_UNKNOWN "Mtodo de autentificacin desconocido"
 #define MSG_PDB_BROKEN "No se puede leer el fichero indexado puredb (o formato antiguo detectado) - Prueba a ejecutar pure-pw mkdb"
 #define MSG_ALIASES_ALIAS "%s es un alias para %s."
@@ -214,3 +215,19 @@
 #define MSG_ALIASES_LIST "Los siguientes alias estn disponibles :"
 #define MSG_PERUSER_MAX "No puedo aceptar ms de %lu conexiones del mismo usuario"
 #define MSG_IPV6_OK "Las conexiones IPv6 tambin son bienvenidas en este servidor"
+#define MSG_TLS_INFO "TLS: Activado %s con %s, %d bits de cifrado"
+#define MSG_TLS_WEAK "TLS: Cifrado demasiado dbil"
+#define MSG_TLS_NEEDED "Lo siento. No se aceptan sesiones sin cifrar en este servidor.\n" \
+    "Por favor, vuelva a conectar utilizando mecanismos de seguridad TLS."
+#define MSG_ILLEGAL_CHARSET "Illegal charset"
+#define MSG_TLS_NO_CTX "TLS: Context not found. Exiting."
+#define MSG_PROT_OK "Data protection level set to \"%s\""
+#define MSG_PROT_PRIVATE_NEEDED "Data connection cannot be opened with this PROT setting."
+#define MSG_PROT_UNKNOWN_LEVEL "Protection level %s not understood. Fallback to \"%s\""
+#define MSG_PROT_BEFORE_PBSZ "PROT must be preceded by a successful PBSZ command"
+#define MSG_WARN_LDAP_USERPASS_EMPTY "LDAP returned no userPassword attribute, check LDAP access rights."
+#define MSG_LDAP_INVALID_AUTH_METHOD "Invalid LDAPAuthMethod in the configuration file. Should be 'bind' or 'password'."
+#define MSG_INVALID_ARGUMENT "Invalid argument: \"%s\""
+#define MSG_READY_TO_PROCEED "Ready to proceed"
+#define MSG_ASCII_MODE_UNSUPPORTED "Transfers in ASCII mode are not supported"
+#define MSG_ASCII_MODE_WARNING "Warning: client is in ASCII mode - Assuming data is plain text"
