@@ -1,3 +1,4 @@
+#define MSG_TLS_CIPHER_FAILED "TLS: Y a une couillasse dans ton TLSCipherSuite '%s'"
 #define MSG_TIMEOUT "Putain mais achete-toi des doigts"
 #define MSG_CAPABILITIES "Ton kernel m'a envoye chier"
 #define MSG_CLIENT_CLOSED_CNX "Le mec s'est casse"
@@ -10,13 +11,13 @@
 #define MSG_ONLY_IPV4V6 "C'est quoi ce protocole de merde ?"
 #define MSG_ONLY_IPV4 "T'as que de l'IPv4, loser"
 #define MSG_TIMEOUT_PARSER "Putain mais tu dors ou quoi ? Degage, bouffon"
-#define MSG_LINE_TOO_LONG "Sale pute, t'as voulu me baiser"
-#define MSG_LOG_OVERFLOW "Sale pute, t'as voulu me baiser"
+#define MSG_LINE_TOO_LONG "Dans tes reves"
+#define MSG_LOG_OVERFLOW "Meme pas en reve"
 #define MSG_GOODBYE "Degage connard, tu m'as balance %llu Ko dans la gueule et pompe %llu Ko."
 #define MSG_DEBUG_COMMAND "Commande"
 #define MSG_IS_YOUR_CURRENT_LOCATION "c'est la ou tu te trouves ducon"
 #define MSG_NOT_LOGGED_IN "Dans tes reves"
-#define MSG_AUTH_UNIMPLEMENTED "Je crypterai pas ta merde"
+#define MSG_AUTH_UNIMPLEMENTED "Je chiffrerai pas ta merde"
 #define MSG_NO_FILE_NAME "Et le nom du fichier, tu me le chies ?"
 #define MSG_NO_DIRECTORY_NAME "File un nom de repertoire, on verra apres"
 #define MSG_NO_RESTART_POINT "Super, essaye encore"
@@ -34,7 +35,7 @@
 #define MSG_TIMEOUT_DATA "Eh, ca fait %lu secondes que je m'emmerde"
 #define MSG_SLEEPING "Fais tourner le techi..."
 #define MSG_ALREADY_LOGGED "Tu t'amuses ?"
-#define MSG_ANY_PASSWORD "Eh, t'as pas une cloppe ?"
+#define MSG_ANY_PASSWORD "Eh, t'as pas une clope ?"
 #define MSG_ANONYMOUS_LOGGED "Salut tete de mort. Si tu peta mon FTP ma parole j'te bute"
 #define MSG_ANONYMOUS_LOGGED_VIRTUAL "Ahah, le bouffon, il est sur un serveur virtuel"
 #define MSG_USER_OK "Eh %s ! File-moi ton code de carte bleue"
@@ -47,7 +48,7 @@
 #define MSG_AUTH_FAILED "Va te faire foutre"
 #define MSG_AUTH_TOOMANY "Degage connard, va jouer ailleurs"
 #define MSG_NO_HOMEDIR "Mon pitbull a bouffe ton home directory"
-#define MSG_NO_HOMEDIR2 "%s s'est fait enculer"
+#define MSG_NO_HOMEDIR2 "%s s'est fait humidifier le rectum"
 #define MSG_START_SLASH "Bon, on va direct en /"
 #define MSG_USER_GROUP_ACCESS "Putain, %s il fait partie de"
 #define MSG_FXP_SUPPORT "Le FXP est active, alors balance de la wawa"
@@ -81,13 +82,13 @@
 #define MSG_PROBABLY_DENIED "(ou alors t'as essayer de me faire un coup de pute)"
 #define MSG_GLOB_READ_ERROR "Ca me fait chier d'evaluer %s"
 #define MSG_GLOB_NO_MATCH "Tu sors ca d'ou ? Y a pas de %s dans %s"
-#define MSG_CHMOD_FAILED "Va te faire enculer et touche pas a %s"
+#define MSG_CHMOD_FAILED "Va te faire pratiquer un coit anal et touche pas a %s"
 #define MSG_CHMOD_SUCCESS "Un chmod sur %s c'est little player quand meme"
 #define MSG_CHMOD_TOTAL_FAILURE "Ahah comment t'as trop l'air d'un con sur ce coup-la"
 #define MSG_ANON_CANT_DELETE "Tu touches pas a ce putain de fichier ou je te butte"
 #define MSG_ANON_CANT_OVERWRITE "Ce fichier, il est cool, alors t'es gentil, tu touches pas"
 #define MSG_DELE_FAILED "T'as pas assez de couilles pour butter %s"
-#define MSG_DELE_SUCCESS "%s s'est fait defoncer la face"
+#define MSG_DELE_SUCCESS "%s%s%s%s s'est fait defoncer la face"
 #define MSG_DELE_TOTAL_FAILURE "Total loser"
 #define MSG_LOAD_TOO_HIGH \
     "Le serveur a une putain de charge de %3.2f .\n" \
@@ -107,19 +108,19 @@
     "T'es un bouffon, ton ratio est de %u:%u .\n" \
     "T'as balance %llu Ko et pompe %llu Ko.\n" \
     "Alors balance de la stuff ou casse-toi."
-#define MSG_NO_MORE_TO_DOWNLOAD "Plus rien a pomper"
+#define MSG_NO_MORE_TO_DOWNLOAD "Plus rien a pepom"
 #define MSG_WINNER "Total respekt"
-#define MSG_KBYTES_LEFT "%.1f Koctets a pomper"
+#define MSG_KBYTES_LEFT "%.1f Ko a pomper"
 #define MSG_ABORTED "Putain mais pourquoi t'as tout arrete ? T'es con ou quoi ?"
 #define MSG_DATA_WRITE_FAILED "Il m'a raccroche a la gueule cet enfoire"
 #define MSG_DATA_READ_FAILED "Putain il s'est barre, l'encule"
 #define MSG_MMAP_FAILED "Le systeme il est fucke, meme le mmap() il marche pas"
 #define MSG_WRITE_FAILED "Une sodo par un ours, ca te dit ?"
-#define MSG_TRANSFER_RATE_M "%.3f secondes (mesuree ici), %.2f Moctets par seconde"
-#define MSG_TRANSFER_RATE_K "%.3f secondes (mesuree ici), %.2f Koctets par seconde"
-#define MSG_TRANSFER_RATE_B "%.3f secondes (mesuree ici), %.2f octets par seconde"
-#define MSG_SPACE_FREE_M "%.1f Moctets dispos pour rajouter de la bonne"
-#define MSG_SPACE_FREE_K "%f Koctets dispos, c'est tout ?"
+#define MSG_TRANSFER_RATE_M "%.3f secondes (mesurees ici), %.2f Mo par seconde"
+#define MSG_TRANSFER_RATE_K "%.3f secondes (mesurees ici), %.2f Ko par seconde"
+#define MSG_TRANSFER_RATE_B "%.3f secondes (mesurees ici), %.2f octets par seconde"
+#define MSG_SPACE_FREE_M "%.1f Mo dispos pour rajouter de la bonne"
+#define MSG_SPACE_FREE_K "%f Ko dispos, c'est tout ?"
 #define MSG_DOWNLOADED "leeche"
 #define MSG_REST_NOT_NUMERIC "REST c'est pas pour les bouffons"
 #define MSG_REST_ASCII_STRICT "C'est quoi ton client de merde ?"
@@ -172,7 +173,7 @@
 #define MSG_NEW_CONNECTION "%s ramene son cul"
 #define MSG_WELCOME_TO "C'est de la bonne :"
 #define MSG_MAX_USERS "%lu relous (le maximum) sont deja connectes, degage"
-#define MSG_NB_USERS "T'es le numero %d sur les %d max."
+#define MSG_NB_USERS "T'es le numero %u sur les %u max."
 #define MSG_WELCOME_TIME "L'heure locale est %02d:%02d. Port du serveur : %u."
 #define MSG_ANONYMOUS_FTP_ONLY "Je prend que les mecs anonymes"
 #define MSG_RATIOS_EVERYONE "Y A DES RATIOS POUR TOUS LES UTILISATEURS :"
@@ -197,7 +198,7 @@
 #define MSG_ILLEGAL_TRUSTED_IP "Fous une IP valide et ca ira mieux"
 #define MSG_NO_ASCII_RESUME "D'abord tu degages le fichier, apres on discute"
 #define MSG_UNKNOWN_ALTLOG "Format d'historique de merde"
-#define MSG_ACCOUNT_DISABLED "Le comte [%s] s'est fait buter"
+#define MSG_ACCOUNT_DISABLED "Le compte [%s] s'est fait buter"
 #define MSG_SQL_WRONG_PARMS "Parametre SQL a la con"
 #define MSG_ILLEGAL_CONFIG_FILE_SQL "Fichier de configuration SQL foireux"
 #define MSG_SQL_MISSING_SERVER "Pas de serveur dans la configuration SQL"
@@ -205,17 +206,28 @@
 #define MSG_ILLEGAL_QUOTA "Quota tout pourri"
 #define MSG_QUOTA_FILES "%llu fichiers utilises (%d%%) - autorises : %llu fichiers"
 #define MSG_QUOTA_SIZE "%llu Ko utilises (%d%%) - autorises : %llu Ko"
-#define MSG_QUOTA_EXCEEDED "T'as explose ton quota : %s va etre pulverise"
+#define MSG_QUOTA_EXCEEDED "T'as explose ton quota : [%s] va etre pulverise"
 #define MSG_AUTH_UNKNOWN "C'est quoi ta methode d'authentification a 2 balles ?"
-#define MSG_PDB_BROKEN "Impossible de lire le fichier puredb indexe (ou il est dans un vieux format) - Essayez pure-pw mkdb"
+#define MSG_PDB_BROKEN "Impossible de lire le fichier puredb indexe (ou il est dans un vieux format pourri) - Tente pure-pw mkdb"
 #define MSG_ALIASES_ALIAS "%s c'est un truc de flemmard a la place de %s."
 #define MSG_ALIASES_UNKNOWN "Je connais pas %s."
 #define MSG_ALIASES_BROKEN_FILE "Ahahah gros naze, ton fichier d'alias est tout pourrave"
 #define MSG_ALIASES_LIST "Si t'as la flemme, tu peux utiliser ces raccourcis :"
 #define MSG_PERUSER_MAX "T'es deja la %lu fois, j'en ai marre de voir ta gueule"
 #define MSG_IPV6_OK "Si t'es pas un batard, tu peux te connecter en IPv6."
-#define MSG_TLS_INFO "SSL/TLS: Direct, le scarla il deale du %s (%s) en %d bits !"
-#define MSG_TLS_WEAK "SSL/TLS: C'est quoi ce chiffrement de tafiole ?"
+#define MSG_TLS_INFO "TLS: Direct, le scarla il deale du %s (%s) en %d bits !"
+#define MSG_TLS_WEAK "TLS: C'est quoi ce chiffrement de tafiole ?"
 #define MSG_TLS_NEEDED "Ecoute-moi connard : t'as 10 secondes pour configurer\n" \
-    "ton client de merde en SSL/TLS. En attendant, degage."
+    "ton client de merde en TLS. En attendant, degage."
 #define MSG_ILLEGAL_CHARSET "Ton jeu de caractere il puxore grave."
+#define MSG_TLS_NO_CTX "TLS: contexte introuvable. Cassos."
+#define MSG_PROT_OK "Niveau de super mega protection : \"%s\""
+#define MSG_PROT_PRIVATE_NEEDED "Le reglage envoye avec PROT est une grosse daube que je refuse pour transmettre des donnees."
+#define MSG_PROT_UNKNOWN_LEVEL "C'est quoi ce niveau de protection %s ? Pas compris. Je vais donc choisir \"%s\""
+#define MSG_PROT_BEFORE_PBSZ "PROT ca marche qu'apres un PBSZ reussi"
+#define MSG_WARN_LDAP_USERPASS_EMPTY "LDAP n'a pas retourne d'attribut userPassword, ptet que les droits d'acces LDAP chient."
+#define MSG_LDAP_INVALID_AUTH_METHOD "LDAPAuthMethod dans le fichier de conf, ca doit etre 'bind' ou 'password'."
+#define MSG_INVALID_ARGUMENT "\"%s\" c'est n'importe nawak"
+#define MSG_READY_TO_PROCEED "OP"
+#define MSG_ASCII_MODE_UNSUPPORTED "Tu veux faire un transfert en ASCII ? T'es un malade toi"
+#define MSG_ASCII_MODE_WARNING "Attention : ton client est configure en ASCII. T'es serieux, la ? Tu vas te retrouver avec un fichier tout pete"
