@@ -1,3 +1,4 @@
+#define MSG_TLS_CIPHER_FAILED "TLS: Invalid TLSCipherSuite specified '%s'"
 #define MSG_TIMEOUT "Onderbreking"
 #define MSG_CAPABILITIES "Onmogelijk om de instelling te wisselen"
 #define MSG_CLIENT_CLOSED_CNX "Client sloot de verbinding"
@@ -87,7 +88,7 @@
 #define MSG_ANON_CANT_DELETE "Anonieme gebruikers kunnen geen bestanden wissen"
 #define MSG_ANON_CANT_OVERWRITE "Anonieme gebruikers kunnen geen bestaande bestanden overschrijven."
 #define MSG_DELE_FAILED "Kon %s niet wissen"
-#define MSG_DELE_SUCCESS "%s gewist"
+#define MSG_DELE_SUCCESS "%s%s%s%s gewist"
 #define MSG_DELE_TOTAL_FAILURE "Geen bestand gewist"
 #define MSG_LOAD_TOO_HIGH \
     "De belasting was %3.2f toen u verbinding kreeg. Wij staan geen\n" \
@@ -172,7 +173,7 @@
 #define MSG_NEW_CONNECTION "Nieuwe verbinding vanaf %s"
 #define MSG_WELCOME_TO "Welkom bij"
 #define MSG_MAX_USERS "%lu gebruikers (het maximum) zijn reeds ingelogd, sorry"
-#define MSG_NB_USERS "U bent gebruiker nummer %d van %d toegestane gebruikers"
+#define MSG_NB_USERS "U bent gebruiker nummer %u van %u toegestane gebruikers"
 #define MSG_WELCOME_TIME "Locale tijd is nu %02d:%02d. Server poort: %u."
 #define MSG_ANONYMOUS_FTP_ONLY "Alleen anonieme FTP is hier toegestaan"
 #define MSG_RATIOS_EVERYONE "UL/DL VERHOUDINGEN ZIJN VOOR IEDEREEN INGESCHAKELD:"
@@ -214,15 +215,19 @@
 #define MSG_ALIASES_LIST "De volgende aliassen zijn beschikbaar :"
 #define MSG_PERUSER_MAX "Ik kan niet meer dan %lu verbindingen van dezelfde gebruiker accepteren"
 #define MSG_IPV6_OK "IPv6 verbindingen zijn ook welkom op deze server"
-#define MSG_TLS_INFO "SSL/TLS: Enabled %s with %s, %d secret bits cipher"
-#define MSG_TLS_WEAK "SSL/TLS: Cipher too weak"
+#define MSG_TLS_INFO "TLS: Enabled %s with %s, %d secret bits cipher"
+#define MSG_TLS_WEAK "TLS: Cipher too weak"
 #define MSG_TLS_NEEDED "Sorry, cleartext sessions are not accepted on this server.\n" \
-    "Please reconnect using SSL/TLS security mechanisms."
+    "Please reconnect using TLS security mechanisms."
 #define MSG_ILLEGAL_CHARSET "Illegal charset"
-#define MSG_TLS_NO_CTX "SSL/TLS: Context not found. Exiting."
+#define MSG_TLS_NO_CTX "TLS: Context not found. Exiting."
 #define MSG_PROT_OK "Data protection level set to \"%s\""
 #define MSG_PROT_PRIVATE_NEEDED "Data connection cannot be opened with this PROT setting."
 #define MSG_PROT_UNKNOWN_LEVEL "Protection level %s not understood. Fallback to \"%s\""
 #define MSG_PROT_BEFORE_PBSZ "PROT must be preceded by a successful PBSZ command"
 #define MSG_WARN_LDAP_USERPASS_EMPTY "Geen userPassword attribuut aangetroffen. Controleer de toegangsrechten."
 #define MSG_LDAP_INVALID_AUTH_METHOD "Onjuiste LDAPAuthMethod in de configuratie. Moet 'bind' of 'password' zijn."
+#define MSG_INVALID_ARGUMENT "Invalid argument: \"%s\""
+#define MSG_READY_TO_PROCEED "Ready to proceed"
+#define MSG_ASCII_MODE_UNSUPPORTED "Transfers in ASCII mode are not supported"
+#define MSG_ASCII_MODE_WARNING "Warning: client is in ASCII mode - Assuming data is plain text"
