@@ -1,4 +1,4 @@
-#define MSG_TLS_CIPHER_FAILED "SSL/TLS: Invalid TLSCipherSuite specified '%s'"
+#define MSG_TLS_CIPHER_FAILED "TLS: Invalid TLSCipherSuite specified '%s'"
 #define MSG_TIMEOUT "Timeout"
 #define MSG_CAPABILITIES "Kan ikke endre egenskaper"
 #define MSG_CLIENT_CLOSED_CNX "Klienten lukket tilkoblingen"
@@ -173,7 +173,7 @@
 #define MSG_NEW_CONNECTION "Ny tilkobling fra %s"
 #define MSG_WELCOME_TO "Velkommen til"
 #define MSG_MAX_USERS "%lu brukere (maks) er allerede logget p"
-#define MSG_NB_USERS "Du er bruker nummer %d av %d tillatte"
+#define MSG_NB_USERS "Du er bruker nummer %u av %u tillatte"
 #define MSG_WELCOME_TIME "Den lokale tiden er n %02d:%02d. Server port: %u."
 #define MSG_ANONYMOUS_FTP_ONLY "Kun anonym FTP er tillatt her"
 #define MSG_RATIOS_EVERYONE "RATIO ER SATT P FOR ALLE:"
@@ -215,15 +215,19 @@
 #define MSG_ALIASES_LIST "Flgende alias er tilgjengelige:"
 #define MSG_PERUSER_MAX "Kan ikke akseptere mer enn %lu tilkoblinger fra samme bruker"
 #define MSG_IPV6_OK "IPv6-forbindelser er ogs velkomne p denne serveren"
-#define MSG_TLS_INFO "SSL/TLS: Aktivert %s med %s, %d bits kryperingsnkkel"
-#define MSG_TLS_WEAK "SSL/TLS: Krypteringsnkkelen er for svak"
+#define MSG_TLS_INFO "TLS: Aktivert %s med %s, %d bits kryperingsnkkel"
+#define MSG_TLS_WEAK "TLS: Krypteringsnkkelen er for svak"
 #define MSG_TLS_NEEDED "Beklager, klartekst-forbindelser er ikke tillatt p denne serveren.\n" \
-    "Vr vennlig  koble til p nytt ved bruk av SSL/TLS-sikkerhetsmekanismer."
+    "Vr vennlig  koble til p nytt ved bruk av TLS-sikkerhetsmekanismer."
 #define MSG_ILLEGAL_CHARSET "Illegal charset"
-#define MSG_TLS_NO_CTX "SSL/TLS: Context not found. Exiting."
+#define MSG_TLS_NO_CTX "TLS: Context not found. Exiting."
 #define MSG_PROT_OK "Data protection level set to \"%s\""
 #define MSG_PROT_PRIVATE_NEEDED "Data connection cannot be opened with this PROT setting."
 #define MSG_PROT_UNKNOWN_LEVEL "Protection level %s not understood. Fallback to \"%s\""
 #define MSG_PROT_BEFORE_PBSZ "PROT must be preceded by a successful PBSZ command"
 #define MSG_WARN_LDAP_USERPASS_EMPTY "LDAP returned no userPassword attribute, check LDAP access rights."
 #define MSG_LDAP_INVALID_AUTH_METHOD "Invalid LDAPAuthMethod in the configuration file. Should be 'bind' or 'password'."
+#define MSG_INVALID_ARGUMENT "Invalid argument: \"%s\""
+#define MSG_READY_TO_PROCEED "Ready to proceed"
+#define MSG_ASCII_MODE_UNSUPPORTED "Transfers in ASCII mode are not supported"
+#define MSG_ASCII_MODE_WARNING "Warning: client is in ASCII mode - Assuming data is plain text"
