@@ -1,3 +1,4 @@
+#define MSG_TLS_CIPHER_FAILED "TLS: Invalid TLSCipherSuite specified '%s'"
 #define MSG_TIMEOUT "Temporizare expirata"
 #define MSG_CAPABILITIES "Nu pot comuta capabilitatile"
 #define MSG_CLIENT_CLOSED_CNX "Clientul a inchis conexiunea"
@@ -87,7 +88,7 @@
 #define MSG_ANON_CANT_DELETE "Utilizatorii anonimi nu pot sterge fisiere"
 #define MSG_ANON_CANT_OVERWRITE "Utilizatorii anonimi nu pot suprascrie fisiere existente"
 #define MSG_DELE_FAILED "Nu pot sterge %s"
-#define MSG_DELE_SUCCESS "Am sters %s"
+#define MSG_DELE_SUCCESS "Am sters %s%s%s%s"
 #define MSG_DELE_TOTAL_FAILURE "Nici un fisier selectat"
 #define MSG_LOAD_TOO_HIGH \
     "Incarcarea server-ului era %3.2f cind v-ati conectat. Nu sint permise download-uri\n" \
@@ -172,7 +173,7 @@
 #define MSG_NEW_CONNECTION "Conexiune noua de la %s"
 #define MSG_WELCOME_TO "Bine ati venit pe"
 #define MSG_MAX_USERS "%lu utilizatori (maximul) sint deja logati"
-#define MSG_NB_USERS "Sineti utilizatorul cu numarul %d din %d permisi"
+#define MSG_NB_USERS "Sineti utilizatorul cu numarul %u din %u permisi"
 #define MSG_WELCOME_TIME "Timpul local este acum %02d:%02d. Port server: %u."
 #define MSG_ANONYMOUS_FTP_ONLY "Aici este permis numai acces FTP anonim"
 #define MSG_RATIOS_EVERYONE "COTELE SINT ACTIVATE PENTRU ORICINE :"
@@ -205,7 +206,7 @@
 #define MSG_ILLEGAL_QUOTA "Cota eronata"
 #define MSG_QUOTA_FILES "%llu fisiere utilizate (%d%%) - autorizat: %llu fisiere"
 #define MSG_QUOTA_SIZE "%llu Kocteti utilizati (%d%%) - autorizat: %llu Kocteti"
-#define MSG_QUOTA_EXCEEDED "Cota depasita: %s nu va fi salvat"
+#define MSG_QUOTA_EXCEEDED "Cota depasita: [%s] nu va fi salvat"
 #define MSG_AUTH_UNKNOWN "Metoda de autentificare necunoscuta"
 #define MSG_PDB_BROKEN "Nu pot citi fisierul puredb indexat (probabil e in format vechi) - Incercati pure-pw mkdb"
 #define MSG_ALIASES_ALIAS "%s este un alias pentru %s."
@@ -214,3 +215,19 @@
 #define MSG_ALIASES_LIST "Sint disponibile urmatoarele aliasuri: "
 #define MSG_PERUSER_MAX "Nu pot accepta mai mult de %lu conexiuni de la acelasi utilizator"
 #define MSG_IPV6_OK "Conexiunile IPv6 sint acceptate de catre acest server"
+#define MSG_TLS_INFO "TLS: Am activat %s cu %s, criptare cu %d biti"
+#define MSG_TLS_WEAK "TLS: Sistem de criptare prea slab"
+#define MSG_TLS_NEEDED "Sesiunile in text clar nu sint acceptate pe acest server.\n" \
+    "Reconectati-va utilizind mecanismele de securitate TLS."
+#define MSG_ILLEGAL_CHARSET "Illegal charset"
+#define MSG_TLS_NO_CTX "TLS: Context not found. Exiting."
+#define MSG_PROT_OK "Data protection level set to \"%s\""
+#define MSG_PROT_PRIVATE_NEEDED "Data connection cannot be opened with this PROT setting."
+#define MSG_PROT_UNKNOWN_LEVEL "Protection level %s not understood. Fallback to \"%s\""
+#define MSG_PROT_BEFORE_PBSZ "PROT must be preceded by a successful PBSZ command"
+#define MSG_WARN_LDAP_USERPASS_EMPTY "LDAP returned no userPassword attribute, check LDAP access rights."
+#define MSG_LDAP_INVALID_AUTH_METHOD "Invalid LDAPAuthMethod in the configuration file. Should be 'bind' or 'password'."
+#define MSG_INVALID_ARGUMENT "Invalid argument: \"%s\""
+#define MSG_READY_TO_PROCEED "Ready to proceed"
+#define MSG_ASCII_MODE_UNSUPPORTED "Transfers in ASCII mode are not supported"
+#define MSG_ASCII_MODE_WARNING "Warning: client is in ASCII mode - Assuming data is plain text"
