@@ -1,3 +1,4 @@
+#define MSG_TLS_CIPHER_FAILED "TLS: Invalid TLSCipherSuite specified '%s'"
 #define MSG_TIMEOUT "  "
 #define MSG_CAPABILITIES "   (capabilities)"
 #define MSG_CLIENT_CLOSED_CNX "  "
@@ -87,7 +88,7 @@
 #define MSG_ANON_CANT_DELETE "    "
 #define MSG_ANON_CANT_OVERWRITE "     "
 #define MSG_DELE_FAILED "  %s"
-#define MSG_DELE_SUCCESS ": %s"
+#define MSG_DELE_SUCCESS ": %s%s%s%s"
 #define MSG_DELE_TOTAL_FAILURE "    "
 #define MSG_LOAD_TOO_HIGH \
     "       %3.2f.    \n" \
@@ -172,7 +173,7 @@
 #define MSG_NEW_CONNECTION "   %s"
 #define MSG_WELCOME_TO "  "
 #define MSG_MAX_USERS "%lu  ( )  , "
-#define MSG_NB_USERS "  %d  %d "
+#define MSG_NB_USERS "  %u  %u "
 #define MSG_WELCOME_TIME "  %02d:%02d.  : %u."
 #define MSG_ANONYMOUS_FTP_ONLY "     "
 #define MSG_RATIOS_EVERYONE "    :"
@@ -205,7 +206,7 @@
 #define MSG_ILLEGAL_QUOTA " "
 #define MSG_QUOTA_FILES "%llu   (%d%%) - : %llu "
 #define MSG_QUOTA_SIZE "%llu   (%d%%) - : %llu "
-#define MSG_QUOTA_EXCEEDED " : %s   "
+#define MSG_QUOTA_EXCEEDED " : [%s]   "
 #define MSG_AUTH_UNKNOWN "  "
 #define MSG_PDB_BROKEN "    puredb (   ) -  pure-pw mkdb"
 #define MSG_ALIASES_ALIAS "%s    %s."
@@ -214,7 +215,19 @@
 #define MSG_ALIASES_LIST "  :"
 #define MSG_PERUSER_MAX "    %lu    "
 #define MSG_IPV6_OK "  IPv6     "
-#define MSG_TLS_INFO "SSL/TLS:  %s  %s, %d-  "
-#define MSG_TLS_WEAK "SSL/TLS:  "
+#define MSG_TLS_INFO "TLS:  %s  %s, %d-  "
+#define MSG_TLS_WEAK "TLS:  "
 #define MSG_TLS_NEEDED ",     .\n" \
-     "  ,   SSL/TLS."
+     "  ,   TLS."
+#define MSG_ILLEGAL_CHARSET "Illegal charset"
+#define MSG_TLS_NO_CTX "TLS: Context not found. Exiting."
+#define MSG_PROT_OK "Data protection level set to \"%s\""
+#define MSG_PROT_PRIVATE_NEEDED "Data connection cannot be opened with this PROT setting."
+#define MSG_PROT_UNKNOWN_LEVEL "Protection level %s not understood. Fallback to \"%s\""
+#define MSG_PROT_BEFORE_PBSZ "PROT must be preceded by a successful PBSZ command"
+#define MSG_WARN_LDAP_USERPASS_EMPTY "LDAP returned no userPassword attribute, check LDAP access rights."
+#define MSG_LDAP_INVALID_AUTH_METHOD "Invalid LDAPAuthMethod in the configuration file. Should be 'bind' or 'password'."
+#define MSG_INVALID_ARGUMENT "Invalid argument: \"%s\""
+#define MSG_READY_TO_PROCEED "Ready to proceed"
+#define MSG_ASCII_MODE_UNSUPPORTED "Transfers in ASCII mode are not supported"
+#define MSG_ASCII_MODE_WARNING "Warning: client is in ASCII mode - Assuming data is plain text"
