@@ -1,4 +1,4 @@
-#define MSG_TLS_CIPHER_FAILED "SSL/TLS: Invalid TLSCipherSuite specified '%s'"
+#define MSG_TLS_CIPHER_FAILED "TLS: Invalid TLSCipherSuite specified '%s'"
 #define MSG_TIMEOUT "Cas vyprsal"
 #define MSG_CAPABILITIES "Nemozem zmenit privilegia"
 #define MSG_CLIENT_CLOSED_CNX "Klient zavrel spojenie"
@@ -173,7 +173,7 @@
 #define MSG_NEW_CONNECTION "Nove spojenie z %s"
 #define MSG_WELCOME_TO "Vitajte na"
 #define MSG_MAX_USERS "Lutujem, ale %lu (maximum) pouzivatelov je uz prihlasenych"
-#define MSG_NB_USERS "Ste pouzivatel cislo %d z %d povolenych"
+#define MSG_NB_USERS "Ste pouzivatel cislo %u z %u povolenych"
 #define MSG_WELCOME_TIME "Lokalny cas je teraz %02d:%02d. Serverovy port: %u."
 #define MSG_ANONYMOUS_FTP_ONLY "Tu je povoleny len anonymny pristup"
 #define MSG_RATIOS_EVERYONE "VSETCI POUZIVATELIA MUSIA DODRZAT POMER UL/DL :"
@@ -215,15 +215,19 @@
 #define MSG_ALIASES_LIST "Nasledovne aliasy su dostupne :"
 #define MSG_PERUSER_MAX "Nemozem akceptovat viac ako %lu pripojeni od toho isteho pouzivatela"
 #define MSG_IPV6_OK "Pripojenia pomocou IPv6 su tiez vitane na tomto serveri"
-#define MSG_TLS_INFO "SSL/TLS: Zapnute %s pouziva sifru %s, %d tajnych bitov"
-#define MSG_TLS_WEAK "SSL/TLS: Sifra je prilis slaba"
+#define MSG_TLS_INFO "TLS: Zapnute %s pouziva sifru %s, %d tajnych bitov"
+#define MSG_TLS_WEAK "TLS: Sifra je prilis slaba"
 #define MSG_TLS_NEEDED "Prepacte, ale nesifrovane spojenia nie su povolene na tomto serveri.\n" \
-    "Pri najblizsom spojeni pouzite zabezpecovacie mechanizmy SSL/TLS."
+    "Pri najblizsom spojeni pouzite zabezpecovacie mechanizmy TLS."
 #define MSG_ILLEGAL_CHARSET "Illegal charset"
-#define MSG_TLS_NO_CTX "SSL/TLS: Context not found. Exiting."
+#define MSG_TLS_NO_CTX "TLS: Context not found. Exiting."
 #define MSG_PROT_OK "Data protection level set to \"%s\""
 #define MSG_PROT_PRIVATE_NEEDED "Data connection cannot be opened with this PROT setting."
 #define MSG_PROT_UNKNOWN_LEVEL "Protection level %s not understood. Fallback to \"%s\""
 #define MSG_PROT_BEFORE_PBSZ "PROT must be preceded by a successful PBSZ command"
 #define MSG_WARN_LDAP_USERPASS_EMPTY "LDAP returned no userPassword attribute, check LDAP access rights."
 #define MSG_LDAP_INVALID_AUTH_METHOD "Invalid LDAPAuthMethod in the configuration file. Should be 'bind' or 'password'."
+#define MSG_INVALID_ARGUMENT "Invalid argument: \"%s\""
+#define MSG_READY_TO_PROCEED "Ready to proceed"
+#define MSG_ASCII_MODE_UNSUPPORTED "Transfers in ASCII mode are not supported"
+#define MSG_ASCII_MODE_WARNING "Warning: client is in ASCII mode - Assuming data is plain text"
