@@ -1,3 +1,4 @@
+#define MSG_TLS_CIPHER_FAILED "TLS: Invalid TLSCipherSuite specified '%s'"
 #define MSG_TIMEOUT "Timeout"
 #define MSG_CAPABILITIES "Ofrmgen att ndra mjligheter"
 #define MSG_CLIENT_CLOSED_CNX "Klienten avbrt anslutningen"
@@ -172,7 +173,7 @@
 #define MSG_NEW_CONNECTION "Nyanslutning frn %s"
 #define MSG_WELCOME_TO "Vlkommen till"
 #define MSG_MAX_USERS "%lu anvndare (maximala antalet) r redan inloggade"
-#define MSG_NB_USERS "Du r anvndare nummer %d av %d tilltna"
+#define MSG_NB_USERS "Du r anvndare nummer %u av %u tilltna"
 #define MSG_WELCOME_TIME "Lokal tid r nu %02d:%02d. Server port: %u."
 #define MSG_ANONYMOUS_FTP_ONLY "Endast anonym FTP r tillten"
 #define MSG_RATIOS_EVERYONE "RATIOS r aktiverat fr ALLA:"
@@ -214,15 +215,19 @@
 #define MSG_ALIASES_LIST "Fljande alias r tillgngliga :"
 #define MSG_PERUSER_MAX "Jag kan inte acceptera fler n %lu anslutningar frn en och samma anvndare"
 #define MSG_IPV6_OK "IPv6 anslutningar r ocks vlkomna till denna server"
-#define MSG_TLS_INFO "SSL/TLS: Anvnder %s med %s, %d bitars kryptering"
-#define MSG_TLS_WEAK "SSL/TLS: Krypteringen r fr svag"
+#define MSG_TLS_INFO "TLS: Anvnder %s med %s, %d bitars kryptering"
+#define MSG_TLS_WEAK "TLS: Krypteringen r fr svag"
 #define MSG_TLS_NEEDED "Tyvrr r sessioner i klartext inte tilltna p denna server.\n" \
-    "Vnligen teranslut och anvnd SSL/TLS fr kad skerhet."
+    "Vnligen teranslut och anvnd TLS fr kad skerhet."
 #define MSG_ILLEGAL_CHARSET "Illegal charset"
-#define MSG_TLS_NO_CTX "SSL/TLS: Context not found. Exiting."
+#define MSG_TLS_NO_CTX "TLS: Context not found. Exiting."
 #define MSG_PROT_OK "Data protection level set to \"%s\""
 #define MSG_PROT_PRIVATE_NEEDED "Data connection cannot be opened with this PROT setting."
 #define MSG_PROT_UNKNOWN_LEVEL "Protection level %s not understood. Fallback to \"%s\""
 #define MSG_PROT_BEFORE_PBSZ "PROT must be preceded by a successful PBSZ command"
 #define MSG_WARN_LDAP_USERPASS_EMPTY "LDAP returned no userPassword attribute, check LDAP access rights."
 #define MSG_LDAP_INVALID_AUTH_METHOD "Invalid LDAPAuthMethod in the configuration file. Should be 'bind' or 'password'."
+#define MSG_INVALID_ARGUMENT "Invalid argument: \"%s\""
+#define MSG_READY_TO_PROCEED "Ready to proceed"
+#define MSG_ASCII_MODE_UNSUPPORTED "Transfers in ASCII mode are not supported"
+#define MSG_ASCII_MODE_WARNING "Warning: client is in ASCII mode - Assuming data is plain text"
