@@ -25,8 +25,8 @@ void pure_memzero(void * const pnt, const size_t len)
     while (i < len) {
         pnt_[i++] = 0U;
     }
-}
 # endif
+}
 
 int pure_memcmp(const void * const b1_, const void * const b2_, size_t len)
 {
@@ -45,5 +45,9 @@ int pure_memcmp(const void * const b1_, const void * const b2_, size_t len)
 
 int pure_strcmp(const char * const s1, const char * const s2)
 {
-    return pure_memcmp(s1, s2, strlen(s1) + 1U);
+    const size_t s1_len = strlen(s1);
+    const size_t s2_len = strlen(s2);
+    const size_t len = (s1_len < s2_len) ? s1_len : s2_len;
+
+    return pure_memcmp(s1, s2, len + 1);
 }
